
from flask import json
from behave import given, when, then  # @UnresolvedImport
from superdesk import get_resource_service
from superdesk.tests import get_prefixed_url
from superdesk.utc import utcnow
from datetime import timedelta
from features.steps.steps import apply_placeholders


@given('highlights')
def given_highlights(context):
    with context.app.app_context():
        context.desks = {'name': 'test'}
        get_resource_service('desks').post([context.desks])
        context.highlights = {'name': 'highlight', 'desks': [context.desks['_id']],
                              'auto_insert': 'now-12h'}
        get_resource_service('highlights').post([context.highlights])
        task = {'desk': context.desks['_id']}
        context.items = [
            {'headline': 'item1', 'state': 'in_progress', 'task': task,
             'versioncreated': utcnow() - timedelta(minutes=5)},
            {'headline': 'item2', 'state': 'in_progress', 'task': task,
             'versioncreated': utcnow() - timedelta(hours=8)},
            {'headline': 'old', 'state': 'in_progress', 'task': task,
             'versioncreated': utcnow() - timedelta(days=2)},
        ]
        get_resource_service('archive').post(context.items)
        for item in context.items:
            marks = [{'highlights': context.highlights['_id'], 'marked_item': item['_id']}]
            get_resource_service('marked_for_highlights').post(marks)


@when('we create highlights package')
def when_we_create_highglights_package(context):
    data_text = '{"highlight": "%s", "type": "composite", ' \
                '"task": {"user": "#user._id#", "desk": "#desks._id#"}}' \
                % str(context.highlights['_id'])
    data_text = apply_placeholders(context, data_text)
    url = get_prefixed_url(context.app, '/archive')
    context.response = context.client.post(url, data=data_text, headers=context.headers)


@then('we get new package with items')
def then_we_get_new_package_with_items(context):
    assert context.response.status_code == 201, '%d: %s' % (context.response.status_code, context.response.get_data())
    package = json.loads(context.response.get_data())

    groups = package.get('groups')
    assert len(groups) == 2, 'there should be 2 groups'

    refs = groups[1].get('refs')
    assert len(refs) == 2, 'there should be 2 refs %s' % (refs)
    assert refs[0]['headline'] == 'item1', refs[0]
    assert refs[1]['headline'] == 'item2', refs[1]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.metadata.utils import item_url


class DesksResource(Resource):
    url = 'desks'
    item_url = item_url
    item_methods = ['GET']
    resource_methods = ['GET']
    datasource = {
        'source': 'desks',
        'default_sort': [('name', 1)],
        'projection': {
            # NOTE: since schema is not defined here, setting up a projection explicitly is required,
            # otherwise default `eve` fields (projection) will be applied e.q. `{'_id': 1}`
            # and it will cut off all required data.
            # https://github.com/pyeve/eve/blob/afd573d9254a9a23393f35760e9c515300909ccd/eve/io/base.py#L420
            'desk_metadata': 0
        },
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import superdesk

from unittest import mock
from unittest.mock import MagicMock, ANY
from datetime import timedelta
from bson import ObjectId

from superdesk.tests import TestCase
from superdesk.utc import utcnow

from superdesk.errors import PublishHTTPPushServerError, PublishHTTPPushClientError
from apps.publish.enqueue import enqueue_service
from superdesk.publish.publish_queue import PUBLISHED_IN_PACKAGE
from superdesk.publish import publish_queue
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE
from celery.exceptions import SoftTimeLimitExceeded


class TransmitItemTestCase(TestCase):
    """Tests for the transmit_item() function."""

    def setUp(self):
        try:
            from superdesk.publish.publish_content import transmit_item
        except ImportError:
            self.fail("Could not import function under test (transmit_item).")
        else:
            self.func_under_test = transmit_item

    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    def test_marks_items_as_retrying_in_case_of_failure(self, *mocks):

        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception('Error patching item')

        item_1 = {
            '_id': 'item_1',
            'destination': {},
            'subscriber_id': '1',
            'item_id': 'test',
            'headline': 'test headline',
            'item_version': 4,
            'state': 'pending'
        }

        orig_item = item_1.copy()  # item's original state in DB
        fake_get_service().find_one.return_value = orig_item
        self.func_under_test(item_1['_id'])
        fake_get_service().system_update.assert_called_with('item_1', {'_updated': ANY, 'retry_attempt': 1,
                                                                       'state': 'retrying',
                                                                       'next_retry_attempt_at': ANY}, orig_item)

    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    def test_marks_items_as_retrying_second_time_incase_of_failure(self, *mocks):

        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception('Error patching item')

        item_1 = {
            '_id': 'item_1',
            'destination': {},
            'subscriber_id': '1',
            'item_id': 'test',
            'headline': 'test headline',
            'item_version': 4,
            'state': 'retrying',
            'retry_attempt': 1,
            'next_retry_attempt_at': utcnow() - timedelta(minutes=10)
        }

        orig_item = item_1.copy()  # item's original state in DB
        fake_get_service().find_one.return_value = orig_item
        self.func_under_test(item_1['_id'])

        fake_get_service().system_update.assert_called_with('item_1', {'_updated': ANY, 'retry_attempt': 2,
                                                                       'state': 'retrying',
                                                                       'next_retry_attempt_at': ANY}, orig_item)

    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    def test_marks_items_failed_to_transmit_after_all_retry_attempts(self, *mocks):

        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception('Error patching item')
        self.app.config['MAX_TRANSMIT_RETRY_ATTEMPT'] = 4

        item_1 = {
            '_id': 'item_1',
            'destination': {},
            'subscriber_id': '1',
            'item_id': 'test',
            'headline': 'test headline',
            'item_version': 4,
            'state': 'retrying',
            'retry_attempt': 4,
            'next_retry_attempt_at': utcnow() - timedelta(minutes=10)
        }

        orig_item = item_1.copy()  # item's original state in DB
        fake_get_service().find_one.return_value = orig_item
        self.func_under_test(item_1['_id'])
        fake_get_service().system_update.assert_called_with('item_1', {'_updated': ANY, 'state': 'failed'}, orig_item)

    @mock.patch('superdesk.publish.publish_content.logger')
    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    def test_logs_error_even_when_marking_failed_items_fails(self, *mocks):
        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception('Error patching item')
        fake_get_service().system_update.side_effect = Exception('Update error')

        item_1 = {
            '_id': 'item_1',
            'subscriber_id': '1',
            'destination': {},
            'item_id': 'test',
            'headline': 'test headline',
            'item_version': 4,
            'state': 'pending'
        }

        fake_get_service().find_one.return_value = item_1
        self.func_under_test(item_1['_id'])
        fake_logger = mocks[1]
        expected_msg = 'Failed to set the state for failed publish queue item item_1.'
        fake_logger.error.assert_any_call(expected_msg)

    def test_transmit_failure(self):
        subscriber = {
            '_id': ObjectId('56c11bd78b84bb00b0a1905e'),
            'sequence_num_settings': {'max': 9999, 'min': 1},
            '_etag': 'f16b7eaa566f68b8d2561c811ec694bdf819784d',
            'is_active': True,
            'destinations': [{'delivery_type': 'email'}],
            'email': 'test@test.com',
            'subscriber_type': 'digital',
            'name': 'Test',
        }

        self.app.data.insert('subscribers', [subscriber])

        item_1 = {'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_1', 'item_version': 4,
                  'headline': 'pending headline', 'destination': {'delivery_type': 'email'},
                  'subscriber_id': subscriber['_id'], 'formatted_item': 'test'}

        self.app.data.insert('publish_queue', [item_1])
        self.func_under_test(item_1['_id'])
        failed_item = self.app.data.find_one('publish_queue', req=None, _id=item_1['_id'])
        self.assertEqual(failed_item['state'], 'retrying')
        self.assertEqual(failed_item['retry_attempt'], 1)
        self.assertEqual(failed_item['next_retry_attempt_at'], ANY)

    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    @mock.patch('superdesk.publish.registered_transmitters')
    def test_no_retry_on_http_push_client_error(self, *mocks):
        self.app.config['MAX_TRANSMIT_RETRY_ATTEMPT'] = 4

        item_1 = {
            '_id': 'item_1',
            'destination': {},
            'subscriber_id': '1',
            'item_id': 'test',
            'headline': 'test headline',
            'item_version': 4,
            'state': 'pending'
        }

        orig_item = item_1.copy()  # item's original state in DB

        fake_get_service = mocks[1]
        fake_get_service().find_one.return_value = orig_item

        fake_transmitter = MagicMock()
        fake_transmitter.transmit.side_effect = PublishHTTPPushClientError.httpPushError(Exception('client 4xx'))

        fake_transmitters_list = mocks[0]
        fake_transmitters_list.__getitem__.return_value = fake_transmitter

        self.func_under_test(item_1['_id'])

        fake_get_service().system_update.assert_called_with('item_1', {'_updated': ANY,
                                                                       'state': 'failed'}, orig_item)

    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    @mock.patch('superdesk.publish.registered_transmitters')
    def test_retry_on_http_push_server_error(self, *mocks):
        self.app.config['MAX_TRANSMIT_RETRY_ATTEMPT'] = 4

        item_1 = {
            '_id': 'item_1',
            'destination': {},
            'subscriber_id': '1',
            'item_id': 'test',
            'headline': 'test headline',
            'item_version': 4,
            'state': 'pending'
        }

        orig_item = item_1.copy()  # item's original state in DB

        fake_get_service = mocks[1]
        fake_get_service().find_one.return_value = orig_item

        fake_transmitter = MagicMock()
        fake_transmitter.transmit.side_effect = PublishHTTPPushServerError.httpPushError(Exception('server 5xx'))

        fake_transmitters_list = mocks[0]
        fake_transmitters_list.__getitem__.return_value = fake_transmitter

        self.func_under_test(item_1['_id'])

        fake_get_service().system_update.assert_called_with('item_1', {'_updated': ANY, 'retry_attempt': 1,
                                                                       'state': 'retrying',
                                                                       'next_retry_attempt_at': ANY}, orig_item)


class QueueItemsTestCase(TestCase):
    """Tests for the transmit_items() function."""

    def setUp(self):
        try:
            from superdesk.publish.publish_content import get_queue_items
        except ImportError:
            self.fail("Could not import function under test (transmit_items).")
        else:
            self.func_under_test = get_queue_items
            self.queue_items = [
                {'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_1', 'item_version': 4,
                 'headline': 'pending headline', 'destination': {}},
                {'_id': ObjectId(), 'state': 'retrying', 'item_id': 'item_2', 'item_version': 4,
                 'headline': 'retrying headline', 'retry_attempt': 2,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=30)},
                {'_id': ObjectId(), 'state': 'success', 'item_id': 'item_3', 'item_version': 4,
                 'headline': 'success headline', 'retry_attempt': 4,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'failed', 'item_id': 'item_4', 'item_version': 4,
                 'headline': 'failed headline', 'retry_attempt': 10,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'canceled', 'item_id': 'item_5', 'item_version': 4,
                 'headline': 'canceled headline', 'retry_attempt': 4,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'retrying', 'item_id': 'item_6', 'item_version': 4,
                 'headline': 'retrying headline', 'retry_attempt': 2,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
            ]
            self.app.data.insert('publish_queue', self.queue_items)

    def test_get_queue_items(self):
        items = list(self.func_under_test())
        self.assertEqual(len(items), 1)
        for item in items:
            self.assertIn(item['item_id'], ['item_1'])

    def test_get_retry_queue_items(self):
        items = list(self.func_under_test(True))
        self.assertEqual(len(items), 1)
        for item in items:
            self.assertIn(item['item_id'], ['item_6'])

    def test_get_queue_items_with_retrying_items(self):
        item = self.app.data.find_one('publish_queue', req=None, _id=self.queue_items[1]['_id'])
        self.app.data.update('publish_queue', item.get('_id'),
                             {'next_retry_attempt_at': utcnow() - timedelta(minutes=30)},
                             item)
        items = list(self.func_under_test(True))
        self.assertEqual(len(items), 2)
        self.assertListEqual([item_l['item_id'] for item_l in items], ['item_2', 'item_6'])

    @mock.patch.object(enqueue_service, 'ObjectId')
    @mock.patch.object(enqueue_service, 'get_utc_schedule')
    @mock.patch.object(enqueue_service, 'get_resource_service')
    @mock.patch.object(enqueue_service, 'get_formatter')
    def test_enqueue_dict(self, *mocks):
        get_formatter, get_resource_service, _, _ = mocks
        publish_queue = get_resource_service.return_value
        fake_post = publish_queue.post
        service = enqueue_service.EnqueueService()
        fake_formatter = get_formatter.return_value
        doc_dict = {ITEM_TYPE: CONTENT_TYPE.TEXT, PUBLISHED_IN_PACKAGE: False}
        fake_doc = MagicMock()
        fake_doc.__getitem__ = lambda s, k: doc_dict.get(k, MagicMock())
        fake_doc.get = doc_dict.get
        fake_destination = MagicMock()
        fake_subscriber = MagicMock()
        subs_dict = {'destinations': [fake_destination], 'api_enabled': False}
        fake_subscriber.__getitem__ = lambda s, k: subs_dict.get(k, MagicMock())
        fake_subscriber['destinations'] = [fake_destination]
        subscribers = [fake_subscriber]
        fake_formatter.format.return_value = [{'published_seq_num': 42,
                                               'formatted_item': 'test OK'}]
        service.get_destinations = MagicMock(return_value=fake_subscriber['destinations'])
        service.queue_transmission(fake_doc, subscribers)
        self.assertEqual(len(fake_post.call_args_list), 1)
        self.assertEqual(len(fake_post.call_args_list[0][0][0]), 1)
        doc = fake_post.call_args_list[0][0][0][0]
        self.assertEqual(doc['published_seq_num'], 42)
        self.assertEqual(doc['formatted_item'], 'test OK')

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [{'this_should_not': 'work',
                                               'bad_key': 'value'}]
        service.queue_transmission(fake_doc, subscribers)
        # post should not have been called here,
        # because the dict is lacking the mandatory keys
        self.assertFalse(fake_post.called)

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [(42, 'test tuple OK')]
        service.queue_transmission(fake_doc, subscribers)
        self.assertEqual(len(fake_post.call_args_list), 1)
        self.assertEqual(len(fake_post.call_args_list[0][0][0]), 1)
        doc = fake_post.call_args_list[0][0][0][0]
        self.assertEqual(doc['published_seq_num'], 42)
        self.assertEqual(doc['formatted_item'], 'test tuple OK')

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [(1, "2", 3)]
        service.queue_transmission(fake_doc, subscribers)
        # post should not have been called here,
        # because the tuple should be in (published_seq_num, formatted_item) format
        self.assertFalse(fake_post.called)

    @mock.patch.object(publish_queue, 'app')
    def test_delete_encoded_item(self, fake_app):
        fake_storage = fake_app.storage
        fake_storage_delete = fake_storage.delete
        service = publish_queue.PublishQueueService(backend=MagicMock())
        service.get_from_mongo = MagicMock()
        service.get_from_mongo.return_value = [{'_id': "4567", 'encoded_item_id': 'TEST ID'}]
        service.delete({'_id': "4567"})
        fake_storage_delete.call_args == mock.call('TEST ID')


class TransmitItemTestCase(TestCase):

    @mock.patch('superdesk.publish.registered_transmitters')
    def test_softtimeout_exception(self, *mocks):
        self.app.config['MAX_TRANSMIT_RETRY_ATTEMPT'] = 4
        self.app.config['CELERY_TASK_ALWAYS_EAGER'] = False
        subscriber = {
            '_id': ObjectId('56c11bd78b84bb00b0a1905e'),
            'sequence_num_settings': {'max': 9999, 'min': 1},
            '_etag': 'f16b7eaa566f68b8d2561c811ec694bdf819784d',
            'is_active': True,
            'destinations': [{'delivery_type': 'email'}],
            'email': 'test@test.com',
            'subscriber_type': 'digital',
            'name': 'Test',
        }

        self.app.data.insert('subscribers', [subscriber])

        items = [{'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_1', 'item_version': 4,
                  'headline': 'pending headline', 'destination': {'delivery_type': 'email'},
                  'subscriber_id': subscriber['_id'], 'formatted_item': 'test'},
                 {'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_2', 'item_version': 4,
                  'headline': 'pending headline 2', 'destination': {'delivery_type': 'email'},
                  'subscriber_id': subscriber['_id'], 'formatted_item': 'test'}
                 ]

        self.app.data.insert('publish_queue', items)

        fake_transmitter = MagicMock()
        fake_transmitter.transmit.side_effect = SoftTimeLimitExceeded()

        fake_transmitters_list = mocks[0]
        fake_transmitters_list.__getitem__.return_value = fake_transmitter

        with self.assertRaises(SoftTimeLimitExceeded):
            superdesk.publish.publish_content.transmit_subscriber_items(subscriber.get('_id'), False)
        failed_item = self.app.data.find_one('publish_queue', req=None, _id=items[0].get('_id'))
        self.assertEqual(failed_item['state'], 'retrying')
        pending_item = self.app.data.find_one('publish_queue', req=None, _id=items[1].get('_id'))
        self.assertEqual(pending_item['state'], 'pending')
        self.app.config['CELERY_TASK_ALWAYS_EAGER'] = True



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import arrow
import datetime
import logging

from flask import current_app as app
from superdesk import etree as sd_etree, app, get_resource_service
from superdesk.errors import ParserError
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE
from superdesk.metadata.utils import is_normal_package
from lxml import etree

NITF = 'application/nitf+xml'
NS = {'iptc': 'http://iptc.org/std/NITF/2006-10-18',
      'nitf': 'http://iptc.org/std/NITF/2006-10-18/',
      'xhtml': 'http://www.w3.org/1999/xhtml'}

logger = logging.getLogger(__name__)


def get_content_tag(elem):
    try:
        return elem.tag.rsplit('}')[1]
    except IndexError:
        return elem.tag


class NewsMLTwoFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse if the feed is in NewsML 2 format.
    """

    NAME = 'newsml2'

    label = 'News ML 2.0 Parser'
    # map subject qcode prefix to scheme
    # if value is None, no "scheme" is used, and name comes from qcode
    # if value is not None, <name> element is used instead of qcode
    SUBJ_QCODE_PREFIXES = {
        'subj': None
    }
    missing_voc = None

    def can_parse(self, xml):
        return any([xml.tag.endswith(tag) for tag in ['newsMessage', 'newsItem', 'packageItem']])

    def parse(self, xml, provider=None):
        self.root = xml
        items = []
        try:
            header = self.parse_header(xml)
            for item_set in xml.findall(self.qname('itemSet')):
                for item_tree in item_set:
                    item = self.parse_item(item_tree)
                    item['priority'] = header['priority']
                    items.append(item)
            else:
                if xml.tag.endswith('newsItem') or xml.tag.endswith('packageItem'):
                    item = self.parse_item(xml)
                    item.setdefault('priority', header['priority'])
                    items.append(item)
            return items
        except Exception as ex:
            raise ParserError.newsmlTwoParserError(ex, provider)

    def parse_item(self, tree):
        # config is not accessible during __init__, so we check it here
        if self.__class__.missing_voc is None:
            self.__class__.missing_voc = app.config.get('QCODE_MISSING_VOC', 'continue')
            if self.__class__.missing_voc not in ('reject', 'create', 'continue'):
                logger.warning('Bad QCODE_MISSING_VOC value ({value}) using default ("continue")'
                               .format(value=self.missing_voc))
                self.__class__.missing_voc = 'continue'

        item = dict()
        item['guid'] = tree.attrib['guid'] + ':' + tree.attrib['version']
        item['uri'] = tree.attrib['guid']
        item['version'] = tree.attrib['version']

        self.parse_item_meta(tree, item)
        self.parse_content_meta(tree, item)
        self.parse_rights_info(tree, item)

        if is_normal_package(item):
            self.parse_group_set(tree, item)
        else:
            self.parse_content_set(tree, item)

        return item

    def parse_header(self, tree):
        """Parse header element.

        :param tree:
        :return: dict
        """
        header = tree.find(self.qname('header'))
        priority = 5
        if header is not None:
            priority_elt = header.find(self.qname('priority'))
            if priority_elt is not None:
                priority = self.map_priority(priority_elt.text)

        return {'priority': priority}

    def parse_item_meta(self, tree, item):
        """Parse itemMeta tag"""
        meta = tree.find(self.qname('itemMeta'))
        item[ITEM_TYPE] = meta.find(self.qname('itemClass')).attrib['qcode'].split(':')[1]

        versioncreated_elt = meta.find(self.qname('versionCreated'))
        if versioncreated_elt is not None and versioncreated_elt.text:
            item['versioncreated'] = self.datetime(meta.find(self.qname('versionCreated')).text)
        firstcreated_elt = meta.find(self.qname('firstCreated'))
        if firstcreated_elt is not None and firstcreated_elt.text:
            item['firstcreated'] = self.datetime(firstcreated_elt.text)
        item['pubstatus'] = (meta.find(self.qname('pubStatus')).attrib['qcode'].split(':')[1]).lower()
        item['ednote'] = meta.find(self.qname('edNote')).text if meta.find(self.qname('edNote')) is not None else ''

        embargoed = meta.find(self.qname('embargoed'))
        if embargoed is not None and embargoed.text:
            try:
                item['embargoed'] = self.datetime(embargoed.text)
            except ValueError:
                item['embargoed_text'] = embargoed.text  # store it for inspection
                logger.warning("Can't parse embargoed info '%s' on item '%s'", embargoed.text, item['guid'])

    def parse_content_meta(self, tree, item):
        """Parse contentMeta tag"""
        meta = tree.find(self.qname('contentMeta'))

        def parse_meta_item_text(key, dest=None, elemTree=None):
            if dest is None:
                dest = key

            if elemTree is None:
                elemTree = meta

            elem = elemTree.find(self.qname(key))
            if elem is not None:
                if dest == 'urgency':
                    item[dest] = int(elem.text)
                else:
                    item[dest] = elem.text

        parse_meta_item_text('urgency')
        parse_meta_item_text('slugline')
        parse_meta_item_text('headline')
        parse_meta_item_text('by', 'byline')

        item['slugline'] = item.get('slugline', '')
        item['headline'] = item.get('headline', '')

        try:
            if item[ITEM_TYPE] != CONTENT_TYPE.TEXT and item[ITEM_TYPE] != CONTENT_TYPE.COMPOSITE:
                # only for media item
                item['description_text'] = meta.find(self.qname('description')).text
                item['archive_description'] = item['description_text']
        except AttributeError:
            pass

        try:
            item['language'] = meta.find(self.qname('language')).get('tag')
        except AttributeError:
            pass

        self.parse_content_subject(meta, item)
        self.parse_content_place(meta, item)

        for info_source in meta.findall(self.qname('infoSource')):
            if info_source.get('role', '') == 'cRole:source':
                item['original_source'] = info_source.get('literal')
                break

        item['genre'] = []
        for genre_el in meta.findall(self.qname('genre')):
            for name_el in genre_el.findall(self.qname('name')):
                lang = name_el.get(self.qname("lang", ns='xml'))
                if lang and lang.startswith('en'):
                    item['genre'].append({'name': name_el.text})

        self.parse_authors(meta, item)

        content_created = meta.find(self.qname('contentCreated'))
        if content_created is not None and content_created.text and not item.get('firstcreated'):
            item['firstcreated'] = self.datetime(content_created.text)

        content_updated = meta.find(self.qname('contentModified'))
        if content_updated is not None and content_updated.text and not item.get('versioncreated'):
            item['versioncreated'] = self.datetime(content_updated.text)

        return meta

    def parse_content_subject(self, tree, item):
        """Parse subj type subjects into subject list."""
        item['subject'] = []
        for subject_elt in tree.findall(self.qname('subject')):
            qcode_parts = subject_elt.get('qcode', '').split(':')
            if len(qcode_parts) == 2 and qcode_parts[0] in self.SUBJ_QCODE_PREFIXES:
                scheme = self.SUBJ_QCODE_PREFIXES[qcode_parts[0]]
                if scheme is None:
                    # this is a main subject, we use IPTC qcode
                    try:
                        name = app.subjects[qcode_parts[1]]
                    except KeyError:
                        logger.debug("Subject code {code}' not found".format(code=qcode_parts[1]))
                        continue
                else:
                    # we use the given name if it exists
                    name_elt = subject_elt.find(self.qname('name'))
                    name = name_elt.text if name_elt is not None and name_elt.text else ""
                    try:
                        name = self.getVocabulary(scheme, qcode_parts[1], name)
                    except ValueError:
                        logger.info('Subject element rejected for "{code}"'.format(code=qcode_parts[1]))
                        continue

                subject_data = {
                    'qcode': qcode_parts[1],
                    'name': name
                }
                if scheme:
                    subject_data["scheme"] = scheme
                item['subject'].append(subject_data)

    def parse_content_place(self, tree, item):
        """Parse subject with type="cptType:5" into place list."""
        for subject in tree.findall(self.qname('subject')):
            if subject.get('type', '') == 'cptType:5':
                item['place'] = []
                item['place'].append({'name': self.get_literal_name(subject)})
                broader = subject.find(self.qname('broader'))
                if broader is not None:
                    item['place'].append({'name': self.get_literal_name(broader)})

    def parse_rights_info(self, tree, item):
        """Parse Rights Info tag"""
        info = tree.find(self.qname('rightsInfo'))
        if info is not None:
            item['usageterms'] = getattr(info.find(self.qname('usageTerms')), 'text', '')
            # item['copyrightholder'] = info.find(self.qname('copyrightHolder')).attrib['literal']
            # item['copyrightnotice'] = getattr(info.find(self.qname('copyrightNotice')), 'text', None)

    def parse_group_set(self, tree, item):
        item['groups'] = []
        for group in tree.find(self.qname('groupSet')):
            data = {}
            data['id'] = group.attrib['id']
            data['role'] = group.attrib['role']
            data['refs'] = self.parse_refs(group)
            item['groups'].append(data)

    def parse_refs(self, group_tree):
        refs = []
        for tree in group_tree:
            if 'idref' in tree.attrib:
                refs.append({'idRef': tree.attrib['idref']})
            else:
                ref = {}
                if 'version' in tree.attrib:
                    ref['residRef'] = tree.attrib['residref'] + ':' + tree.attrib['version']
                else:
                    ref['residRef'] = tree.attrib['residref']
                ref['contentType'] = tree.attrib['contenttype']
                ref['itemClass'] = tree.find(self.qname('itemClass')).attrib['qcode']

                for headline in tree.findall(self.qname('headline')):
                    ref['headline'] = headline.text

                refs.append(ref)
        return refs

    def parse_content_set(self, tree, item):
        item['renditions'] = {}
        for content in tree.find(self.qname('contentSet')):
            if content.tag == self.qname('inlineXML'):
                try:
                    item['word_count'] = int(content.attrib['wordcount'])
                except (KeyError, ValueError):
                    pass
                content = self.parse_inline_content(content, item=item)
                item['body_html'] = content.get('content')
                if 'format' in content:
                    item['format'] = content.get('format')
            elif content.tag == self.qname('inlineData'):
                item['body_html'] = content.text
                try:
                    item['word_count'] = int(content.attrib['wordcount'])
                except KeyError:
                    pass
            else:
                rendition = self.parse_remote_content(content)
                item['renditions'][rendition['rendition']] = rendition

    def parse_inline_content(self, tree, item, ns=NS['xhtml']):
        if tree.get('contenttype') == NITF:
            try:
                body_content = tree.xpath('.//nitf:body.content/nitf:block/*', namespaces=NS)
            except AttributeError:
                return {'contenttype': NITF, 'content': ''}
            elements = [etree.tostring(sd_etree.clean_html(e), encoding='unicode', method='html') for e in body_content]
            return {'contenttype': NITF, 'content': '\n'.join(elements)}
        else:
            html = tree.find(self.qname('html', ns))
            if html is None:
                try:
                    ns = tree.nsmap.get(None)  # fallback for missing xmlns
                except AttributeError:
                    ns = None
                html = tree.find(self.qname('html', ns))
            body = html.find(self.qname('body', ns))
            elements = []
            for elem in body:
                if elem.text:
                    tag = get_content_tag(elem)
                    elements.append('<%s>%s</%s>' % (tag, elem.text, tag))

            # If there is a single p tag then replace the line feeds with breaks
            if len(elements) == 1 and get_content_tag(body[0]) == 'p':
                elements[0] = elements[0].replace('\n    ', '</p><p>').replace('\n', '<br/>')

            content = dict()
            content['contenttype'] = tree.attrib['contenttype']
            if len(elements) > 0:
                content['content'] = "\n".join(elements)
            elif body.text:
                content['content'] = '<pre>' + body.text + '</pre>'
                content['format'] = CONTENT_TYPE.PREFORMATTED
            return content

    def parse_remote_content(self, tree):
        content = dict()
        content['residRef'] = tree.attrib.get('residref')
        content['sizeinbytes'] = int(tree.attrib.get('size', '0'))
        content['rendition'] = tree.attrib['rendition'].split(':')[1]
        content['mimetype'] = tree.attrib['contenttype']
        content['href'] = tree.attrib.get('href', None)
        return content

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, '%Y-%m-%dT%H:%M:%S.000Z')
        except (ValueError, TypeError):
            try:
                return arrow.get(string).datetime
            except arrow.parser.ParserError:
                raise ValueError(string)

    def get_literal_name(self, item):
        """Get name for item with fallback to literal attribute if name is not provided."""
        name = item.find(self.qname('name'))
        return name.text if name is not None else item.attrib.get('literal')

    def parse_authors(self, meta, item):
        item['authors'] = []
        for creator in meta.findall(self.qname('creator')):
            name = creator.find(self.qname('name'))
            if name is not None:
                item['authors'].append({
                    'uri': creator.get('uri'),
                    'name': name.text,
                })

    def getVocabulary(self, voc_id, qcode, name):
        """Retrieve vocabulary and accept or reject it

        The vocabulary will be kept if it exists in local vocabularies.
        If it doesn't exist, it will be either created or rejected depending
        on the value of QCODE_MISSING_VOC:
            - if "reject", missing vocabulary are rejected (i.e. ValueError is raised)
            - if "create", a new vocabulary is created
            - if "continue" (default), value is returned but not created (it will be present in the resulting item,
              but the missing vocabulary is not added to SD)
        :param str qcode: qcode to check
        :param str name: name
        :return: value to use for name
        :raise ValueError: value is rejected
        """
        vocabularies_service = get_resource_service('vocabularies')
        voc = vocabularies_service.find_one(req=None, _id=voc_id)
        create = False
        if voc is None:
            create = True
            if self.missing_voc == "reject":
                raise ValueError
            elif self.missing_voc == "create":
                voc = {
                    "_id": voc_id,
                    "field_type": None,
                    "items": [],
                    "type": "manageable",
                    "schema": {"name": {}, "qcode": {}, "parent": {}},
                    "service": {
                            "all": 1
                    },
                    "display_name": voc_id.capitalize(),
                    "unique_field": "qcode",
                }
            elif self.missing_voc == "continue":
                return name
            else:
                raise RuntimeError("Unexpected missing_voc value: {}".format(self.missing_voc))
        try:
            items = voc['items']
        except KeyError:
            logger.warning("Creating missing items for {qcode}".format(qcode=qcode))
            voc['items'] = items = []

        for item in items:
            if item['qcode'] == qcode:
                if item.get("is_active", True):
                    return item.get('name', name)
                else:
                    # the vocabulary exists but is disabled
                    raise ValueError

        items.append({"is_active": True, "name": name, "qcode": qcode})
        if create:
            vocabularies_service.post([voc])
        else:
            vocabularies_service.put(voc_id, voc)
        return name


register_feed_parser(NewsMLTwoFeedParser.NAME, NewsMLTwoFeedParser())



"""
Superdesk SAML Authentication

.. versionadded:: 1.9

Configuration
~~~~~~~~~~~~~

To enable SAML auth module, add ``superdesk.auth.saml`` into :ref:`settings.installed_apps` in settings.
Setting :ref:`settings.secret_key` is also required.

Finally you have to specify :ref:`settings.saml_path` in settings where your ``settings.json`` file and
``certs`` are located. See https://github.com/onelogin/python3-saml#how-it-works for more info.

Service provider config for superdesk in ``settings.json`` file example::

    "sp": {
        "entityId": "http://example.com/api/login/saml_metadata",
        "assertionConsumerService": {
            "url": "http://example.com/api/login/saml?acs",
            "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
        },
        "singleLogoutService": {
            "url": "http://example.com/api/login/saml?sls",
            "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
        },
        "NameIDFormat": "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress",
        "x509cert": "",
        "privateKey": ""
    }

"""

import superdesk
import logging

from urllib.parse import urlparse

from flask import current_app as app, request, redirect, make_response, session, jsonify
from superdesk.auth import auth_user

try:
    from onelogin.saml2.auth import OneLogin_Saml2_Auth
    imported = True
except ImportError:
    imported = False


bp = superdesk.Blueprint('saml', __name__)
logger = logging.getLogger(__name__)


def init_app(app):
    app.client_config['saml_auth'] = False
    if app.config.get('SAML_PATH'):
        assert imported, 'onelogin module is not available'
        app.client_config['saml_auth'] = True
        app.client_config['saml_label'] = app.config['SAML_LABEL']
        superdesk.blueprint(bp, app)


def init_saml_auth(req):
    auth = OneLogin_Saml2_Auth(req, custom_base_path=app.config['SAML_PATH'])
    return auth


def prepare_flask_request(request):
    url_data = urlparse(request.url)
    return {
        'https': 'on' if request.scheme == 'https' else 'off',
        'http_host': request.host,
        'server_port': url_data.port,
        'script_name': request.path,
        'get_data': request.args.copy(),
        'post_data': request.form.copy(),
    }


@bp.route('/login/saml', methods=['GET', 'POST'])
def index():
    req = prepare_flask_request(request)
    auth = init_saml_auth(req)
    errors = []

    if 'slo' in request.args:
        name_id = None
        session_index = None
        if 'samlNameId' in session:
            name_id = session['samlNameId']
        if 'samlSessionIndex' in session:
            session_index = session['samlSessionIndex']
        return redirect(auth.logout(name_id=name_id, session_index=session_index))
    elif 'acs' in request.args or request.form:
        auth.process_response()
        errors = auth.get_errors()
        if len(errors) == 0:
            session['samlUserdata'] = auth.get_attributes()
            session['samlNameId'] = auth.get_nameid()
            session['samlSessionIndex'] = auth.get_session_index()
        else:
            logger.error('SAML %s reason=%s', errors, auth.get_last_error_reason())
            return jsonify({
                'req': req,
                'errors': errors,
                'error_reason': auth.get_last_error_reason(),
            })
    elif 'sls' in request.args:
        def dscb():
            session.clear()
        url = auth.process_slo(delete_session_cb=dscb)
        errors = auth.get_errors()
        if len(errors) == 0:
            if url is not None:
                return redirect(url)

    if session.get('samlNameId'):
        return auth_user(session['samlNameId'])

    return redirect(auth.login())


@bp.route('/login/saml_metadata')
def metadata():
    req = prepare_flask_request(request)
    auth = init_saml_auth(req)
    settings = auth.get_settings()
    metadata = settings.get_sp_metadata()
    errors = settings.validate_metadata(metadata)

    if len(errors) == 0:
        resp = make_response(metadata, 200)
        resp.headers['Content-Type'] = 'text/xml'
    else:
        resp = make_response(', '.join(errors), 500)
    return resp



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from eve.defaults import resolve_default_values
import superdesk
from superdesk import config
from flask import current_app as app
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from superdesk import get_resource_service
from eve.utils import ParsedRequest
from apps.tasks import task_statuses
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE
from apps.archive.archive import SOURCE as ARCHIVE
from flask_babel import _

logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'stages'
    service = StagesService(endpoint_name, backend=superdesk.get_backend())
    StagesResource(endpoint_name, app=app, service=service)


class StagesResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'required': True,
            'minlength': 1,
            'iunique_per_parent': 'desk'
        },
        'description': {
            'type': 'string'
        },
        'working_stage': {
            'type': 'boolean',
            'default': False
        },
        'default_incoming': {
            'type': 'boolean',
            'default': False
        },
        'task_status': {
            'type': 'string',
            'allowed': task_statuses
        },
        'desk_order': {
            'type': 'integer',
            'readonly': True
        },
        'desk': Resource.rel('desks', embeddable=True, required=True),
        'content_expiry': {
            'type': 'integer'
        },
        # if true then desk members can see the item on that stage
        # if false then non-members cannot see the item on that stage
        'is_visible': {
            'type': 'boolean',
            'default': True
        },
        # if true then desk members cannot edit items on the stage
        # if false then desk member can edit items on the stage
        'local_readonly': {
            'type': 'boolean',
            'default': False
        },
        'incoming_macro': {
            'type': 'string'
        },
        'outgoing_macro': {
            'type': 'string'
        },
        'onstage_macro': {
            'type': 'string'
        }
    }

    datasource = {'default_sort': [('desk_order', 1)]}

    privileges = {'POST': 'desks', 'DELETE': 'desks', 'PATCH': 'desks'}


class StagesService(BaseService):
    notification_key = 'stage'

    def on_create(self, docs):
        """Runs on stage create.

        Overriding this to set desk_order and expiry settings. Also, if this stage is defined as either working or
        incoming stage or both then removes the old incoming and working stages.
        """

        for doc in docs:
            desk = doc.get('desk')

            if not desk:
                doc['desk_order'] = 1
                continue

            req = ParsedRequest()
            req.sort = '-desk_order'
            req.max_results = 1
            prev_stage = self.get(req=req, lookup={'desk': doc['desk']})

            if doc.get('content_expiry') == 0:
                doc['content_expiry'] = None

            if prev_stage.count() == 0:
                doc['desk_order'] = 1
            else:
                doc['desk_order'] = prev_stage[0].get('desk_order', 1) + 1

            # if this new one is default then remove the old default
            if doc.get('working_stage', False):
                self.remove_old_default(desk, 'working_stage')

            if doc.get('default_incoming', False):
                self.remove_old_default(desk, 'default_incoming')

    def on_created(self, docs):
        for doc in docs:
            if 'desk' in doc:
                push_notification(self.notification_key,
                                  created=1,
                                  stage_id=str(doc.get(config.ID_FIELD)),
                                  desk_id=str(doc.get('desk')),
                                  is_visible=doc.get('is_visible', True))

            if doc.get('working_stage', False):
                self.set_desk_ref(doc, 'working_stage')

            if doc.get('default_incoming', False):
                self.set_desk_ref(doc, 'incoming_stage')

            if not doc.get('is_visible', True):
                get_resource_service('users').update_stage_visibility_for_users()

    def on_delete(self, doc):
        """
        Checks if deleting the stage would not violate data integrity, raises an exception if it does.

            1. Can't delete the stage if it's an incoming stage or a working stage
            2. The stage must have no documents (spiked or unspiked)
            3. The stage can not be referred to by a ingest routing rule

        :param doc:
        """

        if doc['working_stage'] is True:
            desk_id = doc.get('desk', None)
            if desk_id and superdesk.get_resource_service('desks').find_one(req=None, _id=desk_id):
                raise SuperdeskApiError.preconditionFailedError(message=_('Cannot delete a Working Stage.'))

        if doc['default_incoming'] is True:
            desk_id = doc.get('desk', None)
            if desk_id and superdesk.get_resource_service('desks').find_one(req=None, _id=desk_id):
                raise SuperdeskApiError.preconditionFailedError(message=_('Cannot delete a Incoming Stage.'))

        archive_versions_query = {'task.stage': str(doc[config.ID_FIELD])}
        items = superdesk.get_resource_service('archive_versions').get(req=None, lookup=archive_versions_query)
        if items and items.count():
            raise SuperdeskApiError.preconditionFailedError(
                message=_('Cannot delete stage as it has article(s) or referenced by versions of the article(s).'))

        # check if the stage is referred to in a ingest routing rule
        rules = self._stage_in_rule(doc[config.ID_FIELD])
        if rules.count() > 0:
            rule_names = ', '.join(rule.get('name') for rule in rules)
            raise SuperdeskApiError.preconditionFailedError(
                message=_('Stage is referred by Ingest Routing Schemes : {rule_names}').format(rule_names=rule_names))

    def on_deleted(self, doc):
        push_notification(self.notification_key,
                          deleted=1,
                          stage_id=str(doc.get(config.ID_FIELD)),
                          desk_id=str(doc.get('desk')))

    def on_update(self, updates, original):
        if updates.get('content_expiry') == 0:
            updates['content_expiry'] = None

        super().on_update(updates, original)

        if updates.get('working_stage', False):
            if not original.get('working_stage'):
                self.remove_old_default(original.get('desk'), 'working_stage')
                self.set_desk_ref(original, 'working_stage')
        else:
            if original.get('working_stage') and 'working_stage' in updates:
                raise SuperdeskApiError.forbiddenError(message=_('Must have one working stage in a desk'))

        if updates.get('default_incoming', False):
            if not original.get('default_incoming'):
                self.remove_old_default(original.get('desk'), 'default_incoming')
                self.set_desk_ref(original, 'incoming_stage')
        else:
            if original.get('default_incoming') and 'default_incoming' in updates:
                raise SuperdeskApiError.forbiddenError(message=_('Must have one incoming stage in a desk'))

    def on_updated(self, updates, original):
        if 'is_visible' in updates and updates['is_visible'] != original.get('is_visible', True):
            push_notification('stage_visibility_updated',
                              updated=1,
                              stage_id=str(original[config.ID_FIELD]),
                              desk_id=str(original['desk']),
                              is_visible=updates.get('is_visible', original.get('is_visible', True)))
            get_resource_service('users').update_stage_visibility_for_users()
        else:
            push_notification(self.notification_key,
                              updated=1,
                              stage_id=str(original.get(config.ID_FIELD)),
                              desk_id=str(original.get('desk')))

    def _get_unspiked_stage_documents(self, stage_id):
        """Returns the documents that are on the stage and not spiked.

        :param stage_id:
        :return:
        """
        query_filter = superdesk.json.dumps(
            {'bool': {
                'must': {'term': {'task.stage': stage_id}},
                'must_not': {'term': {ITEM_STATE: CONTENT_STATE.SPIKED}}
            }})
        req = ParsedRequest()
        req.args = {'filter': query_filter}
        return superdesk.get_resource_service(ARCHIVE).get(req, None)

    def get_stage_documents(self, stage_id):
        query_filter = superdesk.json.dumps({'term': {'task.stage': stage_id}})
        req = ParsedRequest()
        req.args = {'filter': query_filter}
        return superdesk.get_resource_service(ARCHIVE).get(req, None)

    def _stage_in_rule(self, stage_id):
        """Returns the ingest routing rules that refer to the passed stage.

        :param stage_id:
        :return: routing scheme rules that refer to the passed stage
        """
        query_filter = {'$or': [{'rules.actions.fetch.stage': str(stage_id)},
                                {'rules.actions.publish.stage': str(stage_id)}]}
        return superdesk.get_resource_service('routing_schemes').get(req=None, lookup=query_filter)

    def get_stages_by_visibility(self, is_visible=False, user_desk_ids=[]):
        """Returns a list of stages for a user.

        """
        if is_visible:
            lookup = {'$or': [{'is_visible': True}]}
            if user_desk_ids:
                lookup['$or'].append({'desk': {'$in': user_desk_ids}})
        else:
            lookup = {'is_visible': False}
            if user_desk_ids:
                lookup['desk'] = {'$nin': user_desk_ids}

        return list(self.get(req=None, lookup=lookup))

    def set_desk_ref(self, doc, field):
        desk = get_resource_service('desks').find_one(_id=doc.get('desk'), req=None)
        if desk:
            get_resource_service('desks').update(doc.get('desk'), {field: doc.get('_id')}, desk)

    def clear_desk_ref(self, doc, field):
        desk = get_resource_service('desks').find_one(_id=doc.get('desk'), req=None)
        if desk:
            get_resource_service('desks').update(doc.get('desk'), {field: None}, desk)

    def remove_old_default(self, desk, field):
        lookup = {'$and': [{field: True}, {'desk': str(desk)}]}
        stages = self.get(req=None, lookup=lookup)
        for stage in stages:
            get_resource_service('stages').update(stage.get('_id'), {field: False}, stage)

    def create_working_stage(self):
        """Creates a Working stage and returns it's identifier

        :return: identifier of Working Stage
        """

        stage = {'name': 'Working Stage', 'working_stage': True, 'desk_order': 1, 'content_expiry': None}
        resolve_default_values(stage, app.config['DOMAIN'][self.datasource]['defaults'])
        return self.create([stage])

    def create_incoming_stage(self):
        """Creates a Incoming stage and returns it's identifier

        :return: identifier of Incoming Stage
        """
        stage = {'name': 'Incoming Stage', 'default_incoming': True, 'desk_order': 2, 'content_expiry': None}
        resolve_default_values(stage, app.config['DOMAIN'][self.datasource]['defaults'])
        return self.create([stage])



"""Geonames utils."""

import requests

from urllib.parse import urljoin
from urllib3.util.retry import Retry

from flask import current_app as app

session = requests.Session()
retries = Retry(total=3, backoff_factor=0.1)
adapter = requests.adapters.HTTPAdapter(max_retries=retries)
session.mount('http://', adapter)
session.mount('https://', adapter)


def get_geonames_state_qcode(place):
    return 'iso3166-2:%s-%s' % (place.get('country_code'), place.get('state_code'))


def get_geonames_country_qcode(place):
    return 'iso3166-1a2:%s' % place.get('country_code')


def get_geonames_qcode(place):
    if place.get('feature_class', '').upper() == 'A':
        if place.get('state_code'):  # state
            return get_geonames_state_qcode(place)
        else:  # country
            return get_geonames_country_qcode(place)
    return 'geonames:%s' % place.get('code', '0')


def format_geoname_item(item):
    geo = {
        'scheme': 'geonames',
        'code': str(item['geonameId']),
        'name': item['name'],
        'state': item.get('adminName1'),
        'region': item.get('adminName2'),
        'country': item.get('countryName'),
        'state_code': item.get('adminCode1'),
        'region_code': item.get('adminCode2'),
        'country_code': item.get('countryCode'),
        'continent_code': item.get('continentCode'),
        'feature_class': item.get('fcl'),
        'location': {
            'lat': float(item['lat']),
            'lon': float(item['lng']),
        }
    }

    if item.get('timezone'):
        geo['tz'] = item['timezone'].get('timeZoneId')

    return geo


def geonames_request(service, service_params):
    params = [
        ('type', 'json'),
        ('username', app.config.get('GEONAMES_USERNAME', '')),
    ]

    if app.config.get('GEONAMES_TOKEN'):
        params.append(('token', app.config['GEONAMES_TOKEN']))

    params.extend(service_params)
    url = urljoin(app.config['GEONAMES_URL'], service)
    res = session.get(url, params=params, timeout=10)
    if res.status_code != 200:
        res.raise_for_status()
    return res.json()



import socket
import logging
from urllib.parse import urljoin
from flask import current_app as app
from superdesk.errors import SuperdeskApiError

import requests
from bson import json_util
from urllib.parse import urlparse

logger = logging.getLogger(__name__)


class VideoEditorFactory():
    """Base class for Video Editor.
    """

    def get(self, project_id):
        """Get single project.

        :param project_id:
        """
        raise NotImplementedError

    def get_paginate(self, page):
        """Get list of projects.

        :param page:
        """
        raise NotImplementedError

    def post(self, file_storage):
        """Create new project.

        :param file_storage:
        """
        raise NotImplementedError

    def duplicate(self, project_id):
        """Duplicate video. This method clone original project and increase version.

        :param project_id: id project.
        :param updates: changes apply to the video.
        """
        raise NotImplementedError

    def put(self, project_id, updates):
        """Edit video. This method does not create a new project.

        :param project_id: id project.
        :param updates: changes apply to the video.
        """
        raise NotImplementedError

    def delete(self, project_id):
        """Delete video in video server.

        :param project_id:
        """
        raise NotImplementedError

    def get_timeline_thumbnails(self, project_id, amount):
        """Get video thumbnails.

        :param project_id:
        :param amount: number of thumbnails to generate/get
        """
        raise NotImplementedError

    def post_preview_thumbnail(self, project_id, file):
        """Upload video preview thumbnail to video server.

        :param project_id:
        :param post_type: Post preview thumbnail type, either capture or upload
        :param time: time to capture
        :param base64_data: base64 image data if type is upload
        """
        raise NotImplementedError

    def get_preview_thumbnail(self, project_id, position=0, crop=None, rotate=None):
        """Capture thumbnail of video on video-server.

        :param project_id:
        :param time: period time to capture the image in video
        :return:
        """
        raise NotImplementedError

    def check_video_server(self):
        """Check the video-server is exist."""
        raise NotImplementedError


class VideoEditorService(VideoEditorFactory):
    """Video Service for integating with video server"""

    label = 'Video Editor'

    def __init__(self, ):
        self.session = requests.Session()
        self.session.headers.update({'User-Agent': 'superdesk'})

    def get_base_url(self):
        return app.config.get('VIDEO_SERVER_URL') + '/projects/'

    def check_video_server(self):
        parse = urlparse(app.config.get('VIDEO_SERVER_URL'))
        host = parse.hostname
        port = parse.port if parse.port else 80
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = sock.connect_ex((host, port))
        if result == 0:
            return True
        else:
            return False

    def _url(self, _id, resource=''):
        url = urljoin(self.get_base_url(), str(_id))
        if resource:
            return urljoin(url + '/', resource)
        return url

    def _get_response(self, resp, *status):
        json = json_util.loads(resp.text)
        if resp.status_code not in status:
            if resp.status_code == 400:
                raise SuperdeskApiError.badRequestError(message=json)
            if resp.status_code == 409:
                raise SuperdeskApiError.conflictError(message=json)
            if resp.status_code == 404:
                raise SuperdeskApiError.notFoundError(message=json)
            if resp.status_code == 500:
                raise SuperdeskApiError.internalError(message=json)
        return json

    def get(self, project_id):
        try:
            resp = self.session.get(self._url(project_id))
            return self._get_response(resp, 200)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def get_paginate(self, page):
        try:
            resp = self.session.get(self.get_base_url(), params={'page': page})
            return self._get_response(resp, 200)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def post(self, file_storage):
        try:
            video_file = {'file': (file_storage.filename, file_storage.read(), file_storage.mimetype)}
            resp = self.session.post(self.get_base_url(), files=video_file)
            return self._get_response(resp, 201)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def duplicate(self, project_id):
        try:
            resp = self.session.post(self._url(project_id, "duplicate"))
            return self._get_response(resp, 201)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def put(self, project_id, updates):
        try:
            resp = self.session.put(self._url(project_id), json=updates)
            return self._get_response(resp, 202)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def delete(self, project_id):
        try:
            resp = self.session.delete(self._url(project_id))
            if resp.status_code != 204:
                logger.exception(resp.text)
                resp.raise_for_status()
            return True
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def get_timeline_thumbnails(self, project_id, amount):
        try:
            params = {
                'type': 'timeline',
                'amount': amount
            }
            resp = self.session.get(self._url(project_id, 'thumbnails'), params=params)
            return self._get_response(resp, 202)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def post_preview_thumbnail(self, project_id, file):
        try:
            payloads = {
                'file': file,
            }
            resp = self.session.post(self._url(project_id, 'thumbnails'), json=payloads)
            return self._get_response(resp, 200)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)

    def get_preview_thumbnail(self, project_id, position=0, crop=None, rotate=None):
        try:
            params = {
                "type": "preview",
                "position": position
            }
            if crop:
                params["crop"] = crop
            if rotate:
                params["rotate"] = rotate
            resp = self.session.get(self._url(project_id, 'thumbnails'), params=params)
            return self._get_response(resp, 202)
        except ConnectionError as ex:
            raise SuperdeskApiError(message=ex.args[0], status_code=500)



from lxml import etree


from .base import BasePackageElement


class Tags(BasePackageElement):
    """
    As Adobe's idml specification explains:
     ``
     The Tags.xml file contains the XML tag de nitions stored in the InDesign document,
     including unused tags.
     ``
    """

    DEFAULT_TAGS = {
        'Root': {
            'TagColor': 'LightBlue'
        }
    }

    @property
    def filename(self):
        """
        Filename inside IDML package.
        Used as a filename for a file inside zip container.
        :return str: filename
        """
        return 'XML/Tags.xml'

    def _build_etree(self):
        self._etree = etree.Element(
            etree.QName(self.XMLNS_IDPKG, 'Tags'),
            nsmap={'idPkg': self.XMLNS_IDPKG}
        )
        self._etree.set('DOMVersion', self.DOM_VERSION)
        self._add_xmltags()

    def _add_xmltags(self):
        """
        Add XMLTag(s)
        """
        tags = self.merge_attributes(self.DEFAULT_TAGS, self._attributes)

        for tag in tags:
            # XMLTag
            xmltag = etree.SubElement(
                self._etree,
                'XMLTag',
                attrib={
                    'Self': 'XMLTag/{}'.format(tag),
                    'Name': tag,
                }
            )
            # Properties
            properties = etree.SubElement(
                xmltag,
                'Properties'
            )
            # TagColor
            tagcolor = etree.SubElement(
                properties,
                'TagColor',
                attrib={
                    'type': 'enumeration'
                }
            )
            tagcolor.text = tags[tag]['TagColor']



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.tests import TestCase
from superdesk import get_resource_service


class ItemDeleteTestCase(TestCase):
    """Tests that item_version documents are deleted when the items document is deleted"""

    def setUp(self):
        self.app.data.insert('items', [
            {'_id': 'item1', 'type': 'text'},
            {'_id': 'item2', 'type': 'text'},
            {'_id': 'item3', 'type': 'text'},
            {'_id': 'item4', 'type': 'text'},
            {'_id': 'item5', 'type': 'text'}
        ])
        self.app.data.insert('items_versions', [
            {'_id': 'ver1', 'type': 'text', '_id_document': 'item1'},
            {'_id': 'ver2', 'type': 'text', '_id_document': 'item2'},
            {'_id': 'ver3', 'type': 'text', '_id_document': 'item3'},
            {'_id': 'ver4', 'type': 'text', '_id_document': 'item4'},
            {'_id': 'ver5', 'type': 'text', '_id_document': 'item5'}
        ])

    def test_event_fired(self):
        items_service = get_resource_service('items')
        items_versions_service = get_resource_service('items_versions')

        item_ids = ['item1', 'item2', 'item3', 'item4', 'item5']
        ver_ids = ['ver1', 'ver2', 'ver3', 'ver4', 'ver5']

        items_service.delete_action(lookup={'_id': {'$in': item_ids}})
        for item_id in item_ids:
            item = items_service.find_one(req=None, _id=item_id)
            self.assertIsNone(item)

        for ver_id in ver_ids:
            item_version = items_versions_service.find_one(req=None, _id=ver_id)
            self.assertIsNone(item_version)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Privileges registry."""
from .errors import PrivilegeNameError

_privileges = {}
_intrinsic_privileges = {}


def privilege(**kwargs):
    """Register privilege.

    Privilege name must not contain "."

    Privilege properties:
    - name
    - label
    - description
    - category
    """
    if '.' in kwargs['name']:
        raise PrivilegeNameError('"." is not supported in privilege name "%s"' % kwargs['name'])
    _privileges[kwargs['name']] = kwargs


def get_privilege_list():
    """Get list of all registered privileges."""
    return [v for v in _privileges.values()]


def intrinsic_privilege(resource_name, method=[]):
    """
    Registers intrinsic privileges.
    """

    _intrinsic_privileges[resource_name] = method


def get_intrinsic_privileges():
    """Get list of all registered intrinsic privileges."""

    return _intrinsic_privileges



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from functools import partial
from unittest.mock import MagicMock, patch
from .utils import mock_dictionaries
import responses
from flask import Flask
from superdesk.tests import TestCase
from superdesk.text_checkers.spellcheckers.base import registered_spellcheckers
from superdesk.text_checkers import spellcheckers
from superdesk.text_checkers.spellcheckers.leuven_dutch import LeuvenDutch, API_URL, OPT_API_KEY
from superdesk import get_resource_service

spellcheckers.AUTO_IMPORT = False
MODEL = {
    'outo': 1,
}


@responses.activate
def load_spellcheckers():
    registered_spellcheckers.clear()
    app = Flask(__name__)
    app.config[OPT_API_KEY] = '123-456-789-ABC'
    spellcheckers.importSpellcheckers(app, spellcheckers.__name__)


class LeuvenDutchTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_spellcheckers()

    def test_list(self):
        """Check that Leuven Dutch spellchecker is listed by spellcheckers_list service"""
        doc = {}
        spellcheckers_list = get_resource_service('spellcheckers_list')
        spellcheckers_list.on_fetched(doc)
        for checker in doc['spellcheckers']:
            if ((checker['name'] == LeuvenDutch.name
                 and 'label' in checker
                 and set(checker['capacities']) == set(LeuvenDutch.capacities)
                 and checker['languages'] == ['nl'])):
                return
        self.fail("Leuven University Dutch spellchecker not found")

    @responses.activate
    def test_checker(self, expected=None, use_internal_dict=False):
        """Check that spellchecking is working

        :param expected: "errors" expected
            to be specified if this test is re-used
        :param use_internal_dict: value to use in request
            can be changed if this test is re-used
        """
        doc = {
            "spellchecker": LeuvenDutch.name,
            "text": "Outo rijden is gevaarlijk",
            "suggestions": False,
            "use_internal_dict": use_internal_dict,
        }
        spellchecker = get_resource_service('spellchecker')
        check_url = API_URL.format(method="spellingchecker")
        responses.add(
            responses.POST, check_url,
            json={
                "status": {
                    "message": "ok",
                    "code": 200
                },
                "settings": {
                    "startmarker": "##__ERR_START__##",
                    "endmarker": "##__ERR_END__##",
                    "html": False
                },
                "spellingchecker": {
                    "input": "Outo rijden is gevaarlijk",
                    "output": {
                        "context": [
                            ""
                        ],
                        "mistakes": [
                            "Outo"
                        ],
                        "marked": "##__ERR_START__##Outo##__ERR_END__## rijden is gevaarlijk"
                    }
                },
                "debug": {
                    "timeneeded": 0.1197
                }
            }
        )
        spellchecker.create([doc])

        if expected is None:
            expected = [{
                'startOffset': 0,
                'text': 'Outo',
                'type': 'spelling'}]

        self.assertEqual(doc['errors'], expected)

    @patch('superdesk.get_resource_service', MagicMock(side_effect=partial(mock_dictionaries, model=MODEL)))
    def test_checker_with_internal_dict(self):
        """Check that words in personal dictionary are discarded correctly

        This test re-uses test_checker but activate "use_internal_dict" option and mock dictionary to have "outo" inside
        """
        # outo is in personal dictionary, so no spelling error should be returned
        expected = []
        self.test_checker(expected=expected, use_internal_dict=True)

    @responses.activate
    def test_suggest(self):
        """Check that spelling suggestions are working"""

        doc = {
            "spellchecker": "leuven_dutch",
            "text": "Outo",
            "suggestions": True
        }
        spellchecker = get_resource_service('spellchecker')
        check_url = API_URL.format(method="suggesties")
        responses.add(
            responses.POST, check_url,
            json={
                "status": {
                    "message": "ok",
                    "code": 200
                },
                "suggesties": {
                    "input": "Outo",
                    "output": {
                        "suggesties": [
                            "Otto",
                            "Buto",
                            "Kato",
                            "Cato",
                            "Lato",
                            "Leto",
                            "Curto",
                            "Mato",
                            "Beto",
                            "Busto"
                        ]
                    }
                },
                "debug": {
                    "timeneeded": 0.0861
                }
            }
        )
        spellchecker.create([doc])

        self.assertEqual(doc, {
            'spellchecker': 'leuven_dutch',
            'suggestions': [{'text': 'Otto'},
                            {'text': 'Buto'},
                            {'text': 'Kato'},
                            {'text': 'Cato'},
                            {'text': 'Lato'},
                            {'text': 'Leto'},
                            {'text': 'Curto'},
                            {'text': 'Mato'},
                            {'text': 'Beto'},
                            {'text': 'Busto'}],
            'text': 'Outo'})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


"""Media archive module"""

import logging

import superdesk
from superdesk.celery_app import celery

from .archive import (
    ArchiveResource, ArchiveService,
    ArchiveVersionsResource, ArchiveVersionsService,
    AutoSaveResource, ArchiveSaveService)
from .commands import RemoveExpiredContent
from .ingest import IngestResource, AppIngestService
from .user_content import UserContentResource, UserContentService
from .archive_lock import ArchiveLockResource, ArchiveUnlockResource, ArchiveLockService, ArchiveUnlockService
from .archive_spike import ArchiveUnspikeResource, ArchiveSpikeService, ArchiveSpikeResource, ArchiveUnspikeService
from .related import ArchiveRelatedResource, ArchiveRelatedService
from apps.common.components.utils import register_component
from apps.item_lock.components.item_lock import ItemLock
from apps.common.models.utils import register_model
from apps.item_lock.models.item import ItemModel
from apps.common.models.io.eve_proxy import EveProxy
from .archive_rewrite import ArchiveRewriteResource, ArchiveRewriteService
from .news import NewsResource, NewsService

logger = logging.getLogger(__name__)


def init_app(app):

    endpoint_name = 'ingest'
    service = AppIngestService(endpoint_name, backend=superdesk.get_backend())
    IngestResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_versions'
    service = ArchiveVersionsService(endpoint_name, backend=superdesk.get_backend())
    ArchiveVersionsResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive'
    service = ArchiveService(endpoint_name, backend=superdesk.get_backend())
    ArchiveResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_lock'
    service = ArchiveLockService(endpoint_name, backend=superdesk.get_backend())
    ArchiveLockResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_unlock'
    service = ArchiveUnlockService(endpoint_name, backend=superdesk.get_backend())
    ArchiveUnlockResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_spike'
    service = ArchiveSpikeService(endpoint_name, backend=superdesk.get_backend())
    ArchiveSpikeResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_unspike'
    service = ArchiveUnspikeService(endpoint_name, backend=superdesk.get_backend())
    ArchiveUnspikeResource(endpoint_name, app=app, service=service)

    endpoint_name = 'user_content'
    service = UserContentService(endpoint_name, backend=superdesk.get_backend())
    UserContentResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_rewrite'
    service = ArchiveRewriteService(endpoint_name, backend=superdesk.get_backend())
    ArchiveRewriteResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_autosave'
    service = ArchiveSaveService(endpoint_name, backend=superdesk.get_backend())
    AutoSaveResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_related'
    service = ArchiveRelatedService(endpoint_name, backend=superdesk.get_backend())
    ArchiveRelatedResource(endpoint_name, app=app, service=service)

    endpoint_name = 'news'
    service = NewsService(endpoint_name, backend=superdesk.get_backend())
    NewsResource(endpoint_name, app=app, service=service)

    from apps.item_autosave.components.item_autosave import ItemAutosave
    from apps.item_autosave.models.item_autosave import ItemAutosaveModel
    register_component(ItemLock(app))
    register_model(ItemModel(EveProxy(superdesk.get_backend())))
    register_component(ItemAutosave(app))
    register_model(ItemAutosaveModel(EveProxy(superdesk.get_backend())))

    superdesk.privilege(name='monitoring_view', label='Monitoring', description='Can access monitoring view.')
    superdesk.privilege(name='archive', label='Archive', description='User can view the published content.')
    superdesk.privilege(name='ingest', label='Ingest', description='User can view content in ingest and fetch it.')
    superdesk.privilege(name='spike', label='Spike', description='Can spike items.')
    superdesk.privilege(name='spike_read', label='Spike - read', description='Can see spiked items.')
    superdesk.privilege(name='unspike', label='Un Spike', description='User can un-spike content.')
    superdesk.privilege(name='unlock', label='Unlock content', description='User can unlock content.')
    superdesk.privilege(name='metadata_uniquename', label='Edit Unique Name', description='User can edit unique name.')
    superdesk.privilege(name='hold', label='Hold', description='Hold a content')
    superdesk.privilege(name='restore', label='Restore', description='Restore a hold a content')
    superdesk.privilege(name='rewrite', label='Update', description='Update a published content')

    superdesk.intrinsic_privilege(ArchiveUnlockResource.endpoint_name, method=['POST'])


@celery.task(soft_time_limit=600)
def content_expiry():
    RemoveExpiredContent().run()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
"""
A module that provides the Superdesk public API application object and runs
the Superdesk public API.

The API is built using the `Eve framework <http://python-eve.org/>`_ and is
thus essentially just a normal `Flask <http://flask.pocoo.org/>`_ application.

.. note:: The public API should not be confused with the "internal" API that
    is meant to be used by the Superdesk browser client only.
"""

import os
import flask
import importlib

from eve import Eve
from eve.io.mongo.mongo import MongoJSONEncoder

from content_api.tokens import SubscriberTokenAuth
from superdesk.datalayer import SuperdeskDataLayer
from superdesk.storage import SuperdeskGridFSMediaStorage
from superdesk.validator import SuperdeskValidator
from superdesk.factory.app import set_error_handlers
from superdesk.factory.sentry import SuperdeskSentry


def get_app(config=None):
    """
    App factory.

    :param dict config: configuration that can override config
        from `settings.py`
    :return: a new SuperdeskEve app instance
    """
    app_config = flask.Config('.')

    # get content api default conf
    app_config.from_object('content_api.app.settings')

    # set some required fields
    app_config.update({'DOMAIN': {'upload': {}}, 'SOURCES': {}})

    try:
        # override from settings module, but only things defined in default config
        import settings as server_settings
        for key in dir(server_settings):
            if key.isupper() and key in app_config:
                app_config[key] = getattr(server_settings, key)
    except ImportError:
        pass  # if exists

    if config:
        app_config.update(config)

    media_storage = SuperdeskGridFSMediaStorage
    if app_config.get('AMAZON_CONTAINER_NAME'):
        from superdesk.storage import AmazonMediaStorage
        media_storage = AmazonMediaStorage

    app = Eve(
        auth=SubscriberTokenAuth,
        settings=app_config,
        data=SuperdeskDataLayer,
        media=media_storage,
        json_encoder=MongoJSONEncoder,
        validator=SuperdeskValidator
    )

    set_error_handlers(app)

    for module_name in app.config.get('CONTENTAPI_INSTALLED_APPS', []):
        app_module = importlib.import_module(module_name)
        try:
            app_module.init_app(app)
        except AttributeError:
            pass

    app.sentry = SuperdeskSentry(app)

    return app


if __name__ == '__main__':
    host = '0.0.0.0'
    port = int(os.environ.get('PORT', '5400'))
    app = get_app()
    app.run(host=host, port=port, debug=True, use_reloader=True)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from operator import itemgetter
from copy import deepcopy
from flask import current_app as app
from eve.utils import config, ParsedRequest
import logging

from eve.versioning import resolve_document_version

from apps.legal_archive.commands import import_into_legal_archive
from apps.legal_archive.resource import LEGAL_PUBLISH_QUEUE_NAME
from apps.publish.content.common import ITEM_KILL
from apps.publish.enqueue import get_enqueue_service
from apps.publish.published_item import published_item_fields, QUEUE_STATE, PUBLISH_STATE, get_content_filter
from apps.packages import PackageService
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.utils import aggregations
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE, not_analyzed, GUID_FIELD, ITEM_STATE, CONTENT_STATE, \
    PUB_STATUS
from superdesk.metadata.packages import LINKED_IN_PACKAGES, PACKAGE, MAIN_GROUP, RESIDREF
from superdesk.notification import push_notification
from apps.archive.common import get_user, item_schema, is_genre, BROADCAST_GENRE, ITEM_OPERATION, is_item_in_package, \
    insert_into_versions
from apps.archive.archive import SOURCE as ARCHIVE
import superdesk
from superdesk.services import BaseService
from superdesk.resource import Resource
from superdesk.utc import utcnow
from apps.publish.content import KillPublishService, TakeDownPublishService
from flask_babel import _

logger = logging.getLogger(__name__)
PACKAGE_TYPE = 'package_type'
TAKES_PACKAGE = 'takes'
SEQUENCE = 'sequence'
LAST_TAKE = 'last_take'


class ArchivedResource(Resource):
    datasource = {
        'search_backend': 'elastic',
        'aggregations': aggregations,
        'default_sort': [('_updated', -1)],
        'elastic_filter_callback': get_content_filter,
        'projection': {
            'old_version': 0,
            'last_version': 0
        }
    }

    mongo_prefix = 'ARCHIVED'

    extra_fields = published_item_fields.copy()
    # item_id + _current_version will be used fetch archived item.
    extra_fields['archived_id'] = {
        'type': 'string',
        'mapping': not_analyzed
    }

    schema = item_schema(extra_fields)
    resource_methods = ['GET']
    item_methods = ['GET', 'PATCH', 'DELETE']
    privileges = {'PATCH': 'archived', 'DELETE': 'archived'}

    additional_lookup = {
        'url': 'regex("[\w,.:-]+")',
        'field': 'archived_id'
    }


class ArchivedService(BaseService):

    def on_create(self, docs):
        package_service = PackageService()

        for doc in docs:
            doc.pop('lock_user', None)
            doc.pop('lock_time', None)
            doc.pop('lock_action', None)
            doc.pop('lock_session', None)
            doc.pop('highlights', None)
            doc.pop('marked_desks', None)
            doc['archived_id'] = self._get_archived_id(doc.get('item_id'), doc.get(config.VERSION))

            if doc.get(ITEM_TYPE) == CONTENT_TYPE.COMPOSITE:
                for ref in package_service.get_item_refs(doc):
                    ref['location'] = 'archived'

    def validate_delete_action(self, doc, allow_all_types=False):
        """Runs on delete of archive item.

        Overriding to validate the item being killed is actually eligible for kill. Validates the following:
            1. Is item of type Text?
            2. Is item a Broadcast Script?
            3. Does item acts as a Master Story for any of the existing broadcasts?
            4. Is item available in production or part of a normal package?
            5. Is the associated Digital Story is available in production or part of normal package?
            6. If item is a Take then is any take available in production or part of normal package?

        :param doc: represents the article in archived collection
        :type doc: dict
        :param allow_all_types: represents if different types of documents are allowed to be killed
        :type doc: bool
        :raises SuperdeskApiError.badRequestError() if any of the above validation conditions fail.
        """

        bad_req_error = SuperdeskApiError.badRequestError

        id_field = doc[config.ID_FIELD]
        item_id = doc['item_id']

        doc['item_id'] = id_field
        doc[config.ID_FIELD] = item_id

        if not allow_all_types and doc[ITEM_TYPE] != CONTENT_TYPE.TEXT:
            raise bad_req_error(message=_('Only Text articles are allowed to be Killed in Archived repo'))

        if is_genre(doc, BROADCAST_GENRE):
            raise bad_req_error(message=_("Killing of Broadcast Items isn't allowed in Archived repo"))

        if get_resource_service('archive_broadcast').get_broadcast_items_from_master_story(doc, True):
            raise bad_req_error(
                message=_("Can't kill as this article acts as a Master Story for existing broadcast(s)"))

        if get_resource_service(ARCHIVE).find_one(req=None, _id=doc[GUID_FIELD]):
            raise bad_req_error(message=_("Can't Kill as article is still available in production"))

        if not allow_all_types and is_item_in_package(doc):
            raise bad_req_error(message=_("Can't kill as article is part of a Package"))

        takes_package_id = self._get_take_package_id(doc)
        if takes_package_id:
            if get_resource_service(ARCHIVE).find_one(req=None, _id=takes_package_id):
                raise bad_req_error(message=_("Can't Kill as the Digital Story is still available in production"))

            req = ParsedRequest()
            req.sort = '[("%s", -1)]' % config.VERSION
            takes_package = list(self.get(req=req, lookup={'item_id': takes_package_id}))
            if not takes_package:
                raise bad_req_error(message=_('Digital Story of the article not found in Archived repo'))

            takes_package = takes_package[0]
            if not allow_all_types and is_item_in_package(takes_package):
                raise bad_req_error(message=_("Can't kill as Digital Story is part of a Package"))

            for takes_ref in self._get_package_refs(takes_package):
                if takes_ref[RESIDREF] != doc[GUID_FIELD]:
                    if get_resource_service(ARCHIVE).find_one(req=None, _id=takes_ref[RESIDREF]):
                        raise bad_req_error(message=_("Can't Kill as Take(s) are still available in production"))

                    take = list(self.get(req=None, lookup={'item_id': takes_ref[RESIDREF]}))
                    if not take:
                        raise bad_req_error(message=_('One of Take(s) not found in Archived repo'))

                    if not allow_all_types and is_item_in_package(take[0]):
                        raise bad_req_error(message=_("Can't kill as one of Take(s) is part of a Package"))

        doc['item_id'] = item_id
        doc[config.ID_FIELD] = id_field

    def on_delete(self, doc):
        self.validate_delete_action(doc)

    def delete(self, lookup):
        if app.testing and len(lookup) == 0:
            super().delete(lookup)
            return

    def command_delete(self, lookup):
        super().delete(lookup)

    def find_one(self, req, **lookup):
        item = super().find_one(req, **lookup)

        if item and str(item.get('task', {}).get('stage', '')) in \
                get_resource_service('users').get_invisible_stages_ids(get_user().get('_id')):
            raise SuperdeskApiError.forbiddenError(_("User does not have permissions to read the item."))

        return item

    def update(self, id, updates, original):
        """Runs on update of archive item.

        Overriding to handle with Kill/Takedown workflow in the Archived repo:
            1. Check if Article has an associated Digital Story and if Digital Story has more Takes.
               If both Digital Story and more Takes exists then all of them would be killed along with the one requested
            2. If the item is flagged as archived only then it was never created by or published from the system so all
                that needs to be done is to delete it and send an email to all subscribers
            3. For each article being killed do the following:
                i.   Create an entry in archive, archive_versions and published collections.
                ii.  Query the Publish Queue in Legal Archive and find the subscribers who received the article
                     previously and create transmission entries in Publish Queue.
                iii. Change the state of the article to Killed in Legal Archive.
                iv.  Delete all the published versions from Archived.
                v.   Send a broadcast email to all subscribers.
        :param id: primary key of the item to be killed
        :type id: str
        :param updates: updates to be applied on the article before saving
        :type updates: dict
        :param original:
        :type original: dict
        """

        # Step 1
        articles_to_kill = self.find_articles_to_kill({'_id': id})
        logger.info('Fetched articles to kill for id: {}'.format(id))
        articles_to_kill.sort(key=itemgetter(ITEM_TYPE), reverse=True)  # Needed because package has to be inserted last
        kill_service = KillPublishService() if updates.get(ITEM_OPERATION) == ITEM_KILL else TakeDownPublishService()

        updated = original.copy()

        for article in articles_to_kill:
            updates_copy = deepcopy(updates)
            kill_service.apply_kill_override(article, updates_copy)
            updated.update(updates_copy)
            # Step 2, If it is flagged as archived only it has no related items in the system so can be deleted.
            # An email is sent to all subscribers
            if original.get('flags', {}).get('marked_archived_only', False):
                super().delete({'item_id': article['item_id']})
                logger.info('Delete for article: {}'.format(article[config.ID_FIELD]))
                kill_service.broadcast_kill_email(article, updates_copy)
                logger.info('Broadcast kill email for article: {}'.format(article[config.ID_FIELD]))
                continue

            # Step 3(i)
            self._remove_and_set_kill_properties(article, articles_to_kill, updated)
            logger.info('Removing and setting properties for article: {}'.format(article[config.ID_FIELD]))

            # Step 3(ii)
            transmission_details = list(
                get_resource_service(LEGAL_PUBLISH_QUEUE_NAME).get(req=None,
                                                                   lookup={'item_id': article['item_id']}))

            if transmission_details:
                get_enqueue_service(updates.get(ITEM_OPERATION, ITEM_KILL)) \
                    .enqueue_archived_kill_item(article, transmission_details)

            article[config.ID_FIELD] = article.pop('item_id', article['item_id'])

            # Step 3(iv)
            super().delete({'item_id': article[config.ID_FIELD]})
            logger.info('Delete for article: {}'.format(article[config.ID_FIELD]))

            # Step 3(i) - Creating entries in published collection
            docs = [article]
            get_resource_service(ARCHIVE).post(docs)
            insert_into_versions(doc=article)
            published_doc = deepcopy(article)
            published_doc[QUEUE_STATE] = PUBLISH_STATE.QUEUED
            get_resource_service('published').post([published_doc])
            logger.info('Insert into archive and published for article: {}'.format(article[config.ID_FIELD]))

            # Step 3(iii)
            import_into_legal_archive.apply_async(countdown=3, kwargs={'item_id': article[config.ID_FIELD]})
            logger.info('Legal Archive import for article: {}'.format(article[config.ID_FIELD]))

            # Step 3(v)
            kill_service.broadcast_kill_email(article, updates_copy)
            logger.info('Broadcast kill email for article: {}'.format(article[config.ID_FIELD]))

    def on_updated(self, updates, original):
        user = get_user()
        push_notification('item:deleted:archived', item=str(original[config.ID_FIELD]),
                          user=str(user.get(config.ID_FIELD)))

    def on_fetched_item(self, doc):
        doc['_type'] = 'archived'

    def _get_archived_id(self, item_id, version):
        return '{}:{}'.format(item_id, version)

    def get_archived_takes_package(self, package_id, take_id, version, include_other_takes=True):
        req = ParsedRequest()
        req.sort = '[("%s", -1)]' % config.VERSION
        take_packages = list(self.get(req=req, lookup={'item_id': package_id}))

        for take_package in take_packages:
            for ref in self._get_package_refs(take_package):
                if ref[RESIDREF] == take_id and (include_other_takes or ref['_current_version'] == version):
                    return take_package

    def find_articles_to_kill(self, lookup, include_other_takes=True):
        """Finds the article to kill.

        If the article is associated with Digital Story then Digital Story will
        also be fetched. If the Digital Story has more takes then all of them would be fetched.

        :param lookup: query to find the main article to be killed
        :type lookup: dict
        :return: list of articles to be killed
        :rtype: list
        """

        archived_doc = self.find_one(req=None, **lookup)
        if not archived_doc:
            return

        req = ParsedRequest()
        req.sort = '[("%s", -1)]' % config.VERSION
        archived_doc = list(self.get(req=req, lookup={'item_id': archived_doc['item_id']}))[0]
        articles_to_kill = [archived_doc]
        takes_package_id = self._get_take_package_id(archived_doc)
        if takes_package_id:
            takes_package = self.get_archived_takes_package(takes_package_id,
                                                            archived_doc['item_id'],
                                                            archived_doc['_current_version'],
                                                            include_other_takes)
            articles_to_kill.append(takes_package)

            if include_other_takes:
                for takes_ref in self._get_package_refs(takes_package):
                    if takes_ref[RESIDREF] != archived_doc[GUID_FIELD]:
                        take = list(self.get(req=req, lookup={'item_id': takes_ref[RESIDREF]}))[0]
                        articles_to_kill.append(take)

        return articles_to_kill

    def _remove_and_set_kill_properties(self, article, articles_to_kill, updates):
        """Removes the irrelevant properties from the given article and sets the properties for kill operation.

        :param article: article from the archived repo
        :type article: dict
        :param articles_to_kill: list of articles which were about to kill from dusty archive
        :type articles_to_kill: list
        :param updates: updates to be applied on the article before saving
        :type updates: dict
        """

        article.pop('archived_id', None)
        article.pop('_type', None)
        article.pop('_links', None)
        article.pop('queue_state', None)
        article.pop(config.ETAG, None)

        for field in ['headline', 'abstract', 'body_html']:
            article[field] = updates.get(field, article.get(field, ''))

        article[ITEM_STATE] = CONTENT_STATE.KILLED if updates[ITEM_OPERATION] == ITEM_KILL else CONTENT_STATE.RECALLED
        article[ITEM_OPERATION] = updates[ITEM_OPERATION]
        article['pubstatus'] = PUB_STATUS.CANCELED
        article[config.LAST_UPDATED] = utcnow()

        user = get_user()
        article['version_creator'] = str(user[config.ID_FIELD])

        resolve_document_version(article, ARCHIVE, 'PATCH', article)

        if article[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
            package_service = PackageService()
            item_refs = package_service.get_item_refs(article)
            for ref in item_refs:
                item_in_package = [item for item in articles_to_kill
                                   if item.get('item_id', item.get(config.ID_FIELD)) == ref[RESIDREF]]
                ref['location'] = ARCHIVE
                ref[config.VERSION] = item_in_package[0][config.VERSION]

    def _get_take_package_id(self, item):
        """Checks if the item is in a 'takes' package and returns the package id

        :return: _id of the package or None
        """
        takes_package = [package.get(PACKAGE) for package in item.get(LINKED_IN_PACKAGES, [])
                         if package.get(PACKAGE_TYPE) == TAKES_PACKAGE]
        if len(takes_package) > 1:
            message = 'Multiple takes found for item: {0}'.format(item[config.ID_FIELD])
            logger.error(message)
        return takes_package[0] if takes_package else None

    def _get_package_refs(self, package):
        """Get refs from the takes package

        :param dict package: takes package
        :return: return refs from the takes package. If not takes package or no refs found then None
        """
        refs = None
        if package:
            groups = package.get('groups', [])
            refs = next((group.get('refs') for group in groups if group['id'] == MAIN_GROUP), None)

        return refs


superdesk.privilege(name='archived', label='Archive Management', description='User can remove items from the archived')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.upload import get_upload_as_data_uri
from flask import current_app as app


bp = superdesk.Blueprint('assets', __name__)


@bp.route('/assets/<path:media_id>', methods=['GET'])
def prod_get_upload_as_data_uri(media_id):
    return get_upload_as_data_uri(media_id)


def upload_url(media_id, view=prod_get_upload_as_data_uri):
    return '{}/{}'.format(
        app.config.get('MEDIA_PREFIX').rstrip('/'),
        media_id
    )


def init_app(app):
    superdesk.blueprint(bp, app)
    app.upload_url = upload_url



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Upload module"""
import logging
import superdesk
from eve.utils import config
from superdesk.errors import SuperdeskApiError
from werkzeug.wsgi import wrap_file
from .resource import Resource
from .services import BaseService
from flask import request, current_app as app, redirect
from superdesk.media.renditions import generate_renditions, delete_file_on_error
from superdesk.media.media_operations import (
    download_file_from_url, download_file_from_encoded_str,
    process_file_from_stream, crop_image, decode_metadata,
)
from superdesk.filemeta import set_filemeta


bp = superdesk.Blueprint('upload_raw', __name__)
logger = logging.getLogger(__name__)
cache_for = 3600 * 24 * 30  # 30d cache


@bp.route('/upload/<path:media_id>/raw', methods=['GET'])
def get_upload_as_data_uri_bc(media_id):
    """Keep previous url for backward compatibility"""
    return redirect(upload_url(media_id))


@bp.route('/upload-raw/<path:media_id>', methods=['GET'])
def get_upload_as_data_uri(media_id):
    if not request.args.get('resource'):
        media_id = app.media.getFilename(media_id)
        media_file = app.media.get(media_id, 'upload')
    else:
        media_file = app.media.get(media_id, request.args['resource'])
    if media_file:
        data = wrap_file(request.environ, media_file, buffer_size=1024 * 256)
        response = app.response_class(
            data,
            mimetype=media_file.content_type,
            direct_passthrough=True)
        response.content_length = media_file.length
        response.last_modified = media_file.upload_date
        response.set_etag(media_file.md5)
        response.cache_control.max_age = cache_for
        response.cache_control.s_max_age = cache_for
        response.cache_control.public = True
        response.make_conditional(request)
        response.headers['Content-Disposition'] = 'inline'
        return response
    raise SuperdeskApiError.notFoundError('File not found on media storage.')


def url_for_media(media_id, mimetype=None):
    return app.media.url_for_media(media_id, mimetype)


def upload_url(media_id, view='upload_raw.get_upload_as_data_uri'):
    media_prefix = app.config.get('MEDIA_PREFIX').rstrip('/')
    return '%s/%s' % (media_prefix, media_id)


def init_app(app):
    endpoint_name = 'upload'
    service = UploadService(endpoint_name, backend=superdesk.get_backend())
    UploadResource(endpoint_name, app=app, service=service)
    superdesk.blueprint(bp, app)
    app.upload_url = upload_url


class UploadResource(Resource):
    schema = {
        'media': {'type': 'file'},
        'CropLeft': {'type': 'integer'},
        'CropRight': {'type': 'integer'},
        'CropTop': {'type': 'integer'},
        'CropBottom': {'type': 'integer'},
        'URL': {'type': 'string'},
        'mimetype': {'type': 'string'},
        'filemeta': {'type': 'dict'}
    }
    extra_response_fields = ['renditions']
    datasource = {
        'projection': {
            'mimetype': 1,
            'filemeta': 1,
            '_created': 1,
            '_updated': 1,
            '_etag': 1,
            'media': 1,
            'renditions': 1,
        }
    }
    item_methods = ['GET', 'DELETE']
    resource_methods = ['GET', 'POST']
    privileges = {'POST': 'archive', 'DELETE': 'archive'}


class UploadService(BaseService):

    def on_create(self, docs):
        for doc in docs:
            if doc.get('URL') and doc.get('media'):
                message = 'Uploading file by URL and file stream in the same time is not supported.'
                raise SuperdeskApiError.badRequestError(message)

            content = None
            filename = None
            content_type = None
            if doc.get('media'):
                content = doc['media']
                filename = content.filename
                content_type = content.mimetype
            elif doc.get('URL'):
                content, filename, content_type = self.download_file(doc)

            self.crop_and_store_file(doc, content, filename, content_type)

    def crop_and_store_file(self, doc, content, filename, content_type):
        # retrieve file name and metadata from file
        file_name, content_type, metadata = process_file_from_stream(content, content_type=content_type)
        # crop the file if needed, can change the image size
        was_cropped, out = crop_image(content, filename, doc)
        # the length in metadata could be updated if it was cropped
        if was_cropped:
            file_name, content_type, metadata_after_cropped = process_file_from_stream(out, content_type=content_type)
            # when cropped, metadata are reseted. Then we update the previous metadata variable
            metadata['length'] = metadata_after_cropped['length']
        try:
            logger.debug('Going to save media file with %s ' % file_name)
            out.seek(0)
            file_id = app.media.put(out, filename=file_name, content_type=content_type,
                                    resource=self.datasource, metadata=metadata)
            doc['media'] = file_id
            doc['mimetype'] = content_type
            set_filemeta(doc, decode_metadata(metadata))
            inserted = [doc['media']]
            file_type = content_type.split('/')[0]
            rendition_spec = config.RENDITIONS['avatar']
            renditions = generate_renditions(out, file_id, inserted, file_type,
                                             content_type, rendition_spec, url_for_media)
            doc['renditions'] = renditions
        except Exception as io:
            for file_id in inserted:
                delete_file_on_error(doc, file_id)
            raise SuperdeskApiError.internalError('Generating renditions failed', exception=io)

    def download_file(self, doc):
        url = doc.get('URL')
        if not url:
            return
        if url.startswith('data'):
            return download_file_from_encoded_str(url)
        else:
            return download_file_from_url(url)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.tests import TestCase
from superdesk.publish import init_app, SUBSCRIBER_TYPES
from superdesk.publish.transmitters.odbc import ODBCPublishService


class ODBCTests(TestCase):
    subscribers = [{"_id": "1", "name": "Test", "subscriber_type": SUBSCRIBER_TYPES.WIRE, "media_type": "media",
                    "is_active": True, "sequence_num_settings": {"max": 10, "min": 1},
                    "critical_errors": {"9004": True},
                    "destinations": [{"name": "AAP IPNEWS", "delivery_type": "odbc", "format": "AAP IPNEWS",
                                      "config": {"stored_procedure": "InsertNews"}
                                      }]
                    }]

    queue_items = [{"_id": "1", "state": "pending", "content_type": "text", "headline": "test", "unique_name": "#2034",
                    "publishing_action": "published", "published_seq_num": 4,
                    "destination": {"name": "AAP IPNEWS", "delivery_type": "odbc", "format": "AAP IPNEWS",
                                    "config": {"stored_procedure": "InsertNews"}
                                    },
                    "formatted_item": {
                        "ident": "0",
                        "selector_codes": '3**',
                        "wordcount": 313,
                        "texttab": "x",
                        "originator": "AAP",
                        "service_level": "a",
                        "keyword": "ROSS",
                        "subject": "crime, law and justice",
                        "category": "a",
                        "take_key": "Take-that",
                        "subject_detail": "international court or tribunal",
                        "subject_reference": "02011001",
                        "article_text": "THIS IS A TEST PLEASE IGNORE",
                        "priority": "u",
                        "headline": "TEST HEADLINE",
                        "usn": 68147,
                        "subject_matter": "international law",
                        "sequence": 117,
                        "news_item_type": "News",
                        "author": "",
                        "genre": "Current",
                        "fullStory": 1
                    },
                    "subscriber_id": "1", "item_id": "1", "item_version": 6
                    }]

    def setUp(self):
        self.subscribers[0]['destinations'][0]['config']['connection_string'] = \
            superdesk.app.config["ODBC_TEST_CONNECTION_STRING"]
        self.app.data.insert('subscribers', self.subscribers)

        self.queue_items[0]['destination']['config']['connection_string'] = \
            superdesk.app.config["ODBC_TEST_CONNECTION_STRING"]
        self.app.data.insert('publish_queue', self.queue_items)
        init_app(self.app)

    def test_transmit(self):
        if superdesk.app.config['ODBC_PUBLISH']:
            subscriber = self.app.data.find('subscribers', None, None)[0]

            publish_service = ODBCPublishService()
            ret = publish_service._transmit(self.queue_items[0], subscriber)
            self.assertGreater(ret, 0)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import logging
import datetime

from copy import deepcopy
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import FeedParser
from superdesk.utc import utc
from superdesk.metadata.utils import generate_tag_from_url

logger = logging.getLogger(__name__)


class NINJSFeedParser(FeedParser):
    """
    Feed Parser for NINJS format
    """

    NAME = 'ninjs'

    label = 'NINJS Feed Parser'

    direct_copy_properties = ('usageterms', 'language', 'headline', 'copyrightnotice',
                              'urgency', 'pubstatus', 'mimetype', 'copyrightholder', 'ednote',
                              'body_text', 'body_html', 'slugline', 'keywords',
                              'extra', 'byline', 'description_text', 'profile')

    items = []

    def __init__(self):
        super().__init__()

    def can_parse(self, file_path):
        try:
            with open(file_path, 'r') as f:
                ninjs = json.load(f)
                if ninjs.get('uri') or ninjs.get('guid'):
                    return True
        except Exception:
            pass
        return False

    def parse(self, file_path, provider=None):
        self.items = []
        with open(file_path, 'r') as f:
            ninjs = json.load(f)

        self.items.append(self._transform_from_ninjs(ninjs))
        return self.items

    def _transform_from_ninjs(self, ninjs):
        guid = ninjs.get('guid')
        if not guid and ninjs.get('uri'):
            guid = generate_tag_from_url(ninjs['uri'], 'urn')
        item = {'guid': guid,
                'type': ninjs.get('type'),
                'uri': ninjs.get('uri')}

        for copy_property in self.direct_copy_properties:
            if ninjs.get(copy_property) is not None:
                item[copy_property] = ninjs[copy_property]

        if ninjs.get('source'):
            item['original_source'] = ninjs['source']

        if ninjs.get('priority'):
            item['priority'] = int(ninjs['priority'])
        else:
            ninjs['priority'] = 5

        if ninjs.get('genre'):
            item['genre'] = self._format_qcodes(ninjs['genre'])

        if ninjs.get('service'):
            item['anpa_category'] = self._format_qcodes(ninjs['service'])

        if ninjs.get('subject'):
            item['subject'] = self._format_qcodes(ninjs['subject'])

        if ninjs.get('versioncreated'):
            item['versioncreated'] = self.datetime(ninjs.get('versioncreated'))

        if ninjs.get('firstcreated'):
            item['firstcreated'] = self.datetime(ninjs.get('firstcreated'))

        if ninjs.get('associations'):
            item['associations'] = {}

        for key, associated_item in ninjs.get('associations', {}).items():
            if associated_item:
                self.items.append(self._transform_from_ninjs(associated_item))
                if associated_item.get('type') == 'picture' and associated_item.get('body_text'):
                    associated_item['alt_text'] = associated_item.get('body_text')
                if associated_item.get('versioncreated'):
                    associated_item['versioncreated'] = self.datetime(associated_item['versioncreated'])
                item['associations'][key] = deepcopy(associated_item)

        if ninjs.get('renditions', {}).get('baseImage'):
            item['renditions'] = {'baseImage': {'href': ninjs.get('renditions', {}).get('original', {}).get('href')}}

        if ninjs.get('located'):
            item['dateline'] = {'located': {'city': ninjs.get('located')}}

        if ninjs.get('type') == 'picture' and ninjs.get('body_text'):
            item['alt_text'] = ninjs.get('body_text')

        if ninjs.get('type') == 'text' and ninjs.get('description_text'):
            item['abstract'] = ninjs.get('description_text')

        if ninjs.get('place'):
            item['place'] = self._format_qcodes(ninjs['place'])

        if ninjs.get('authors'):
            item['authors'] = self._parse_authors(ninjs['authors'])

        if not item.get('body_html') and ninjs.get('body_xhtml'):
            item['body_html'] = ninjs['body_xhtml']

        return item

    def _format_qcodes(self, items):
        subjects = []
        for item in items:
            subject = {'name': item.get('name'), 'qcode': item.get('code')}
            if item.get('scheme'):
                subject['scheme'] = item.get('scheme')
            subjects.append(subject)

        return subjects

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, '%Y-%m-%dT%H:%M:%S+0000').replace(tzinfo=utc)
        except ValueError:
            return datetime.datetime.strptime(string, '%Y-%m-%dT%H:%M:%SZ').replace(tzinfo=utc)

    def _parse_authors(self, authors):
        return [self._parse_author(author) for author in authors]

    def _parse_author(self, author):
        parsed = {
            'name': author['name'],
            'role': author.get('role', ''),
        }

        if author.get('avatar_url'):
            parsed['avatar_url'] = author['avatar_url']

        if author.get('biography'):
            parsed['biography'] = author['biography']

        return parsed


register_feed_parser(NINJSFeedParser.NAME, NINJSFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .macro_register import macros
from superdesk.tests import TestCase


class MacrosTestCase(TestCase):

    def test_register(self):
        with self.app.app_context():
            macros.register(name='test')
            self.assertIn('test', macros)

    def test_load_modules(self):
        with self.app.app_context():
            self.assertIn('usd_to_cad', macros)
            self.assertNotIn('foo name', macros)




import unittest

from .spellcheck import norvig_suggest


class SpellcheckTestCase(unittest.TestCase):

    def test_normal_suggestions(self):
        """Test case for getting the suggestions for the word which aren't name"""
        model = {'foe': 3, 'fox': 5}
        suggestions_1 = norvig_suggest('foo', model)
        self.assertEquals(['fox', 'Fox', 'foe', 'Foe'], suggestions_1)

        suggestions_2 = norvig_suggest('Foo', model)
        self.assertEquals(['fox', 'Fox', 'foe', 'Foe'], suggestions_2)

    def test_name_suggestions(self):
        """Test case for getting the suggestions for name in capitalized case"""

        model = {'Foe': 1, 'fox': 5}
        name_suggestion_1 = norvig_suggest('fooe', model)
        self.assertEquals(['Foe'], name_suggestion_1)

        name_suggestion_2 = norvig_suggest('Fooe', model)
        self.assertEquals(['Foe'], name_suggestion_2)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from urllib.parse import urljoin
import responses
from flask import Flask
from superdesk.tests import TestCase
from superdesk.text_checkers import spellcheckers
from superdesk.text_checkers.spellcheckers.base import registered_spellcheckers
from superdesk.text_checkers.spellcheckers.grammalecte import PATH_CHECK, PATH_SUGGEST, Grammalecte
from superdesk import get_resource_service
import os

spellcheckers.AUTO_IMPORT = False

TEST_URL = "http://localhost:8080"
os.environ['GRAMMALECTE_URL'] = TEST_URL


@responses.activate
def load_spellcheckers():
    """Load spellcheckers by mocking Grammalecte server, so it can be detected"""
    registered_spellcheckers.clear()
    app = Flask(__name__)
    check_url = urljoin(TEST_URL, PATH_CHECK)
    responses.add(
        responses.POST, check_url,
        json={
            "program": "grammalecte-fr",
            "version": "1.2",
        },
    )
    spellcheckers.importSpellcheckers(app, spellcheckers.__name__)


class GrammalecteTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_spellcheckers()

    def test_list(self):
        """Check that Grammalecte is listed by spellcheckers_list service"""
        doc = {}
        spellcheckers_list = get_resource_service('spellcheckers_list')
        spellcheckers_list.on_fetched(doc)
        for checker in doc['spellcheckers']:
            if ((checker['name'] == Grammalecte.name
                 and 'label' in checker
                 and set(checker['capacities']) == set(Grammalecte.capacities)
                 and checker['languages'] == ['fr'])):
                return
        self.fail("Grammalecte not found")

    @responses.activate
    def test_checker(self):
        """Check that spellchecking is working"""
        doc = {
            "spellchecker": "grammalecte",
            "text": "Il nous reste à vérifié votre maquette.",
            "suggestions": False,
            "use_internal_dict": False,
        }
        spellchecker = get_resource_service('spellchecker')
        check_url = urljoin(TEST_URL, PATH_CHECK)
        responses.add(
            responses.POST, check_url,
            json={
                'program': 'grammalecte-fr',
                'version': '1.2',
                'lang': 'fr',
                'error': '',
                'data': [{
                    'iParagraph': 1,
                    'lGrammarErrors': [{
                        'nStart': 14,
                        'nEnd': 15,
                        'sLineId': '#5864',
                        'sRuleId': 'conf_a_à_verbe__b13_a2_0',
                        'sType': 'conf',
                        'aColor': [89, 49, 129],
                        'sMessage': 'Confusion probable : “à” est une préposition. Pour le verbe “avoir”, écrivez “a”.',
                        'aSuggestions': ['a'],
                        'URL': ''},
                        {'nStart': 16,
                         'nEnd': 23,
                         'sLineId': '#5864',
                         'sRuleId': 'conf_a_à_verbe__b13_a3_0',
                         'sType': 'conf',
                         'aColor': [89, 49, 129],
                         'sMessage': 'Le verbe devrait être à l’infinitif.',
                         'aSuggestions': ['vérifier'],
                         'URL': ''}],
                    'lSpellingErrors': []}]
            }
        )
        spellchecker.create([doc])

        expected = [{
            'startOffset': 14,
            'suggestions': [{'text': 'a'}],
            'text': 'à',
            'message': 'Confusion probable : “à” est une préposition. '
                       'Pour le verbe “avoir”, écrivez “a”.',
            'type': 'grammar'},
            {'startOffset': 16,
             'suggestions': [{'text': 'vérifier'}],
             'text': 'vérifié',
             'message': 'Le verbe devrait être à l’infinitif.',
             'type': 'grammar'}]

        self.assertEqual(doc['errors'], expected)

    @responses.activate
    def test_checker_paragraphs(self):
        """Check that spellchecking is working as expected with 2 paragraphs"""
        doc = {
            "spellchecker": "grammalecte",
            "text": "Il nous reste à vérifié votre maquette.\n\nIl nous reste à vérifié votre maquette.",
            "suggestions": False,
            "use_internal_dict": False,
        }
        spellchecker = get_resource_service('spellchecker')
        check_url = urljoin(TEST_URL, PATH_CHECK)
        responses.add(
            responses.POST, check_url,
            json={
                'program': 'grammalecte-fr',
                'version': '1.1',
                'lang': 'fr',
                'error': '',
                'data': [{
                    'iParagraph': 1,
                    'lGrammarErrors': [{
                        'URL': '',
                        'aColor': [89, 49, 129],
                        'aSuggestions': ['a'],
                        'nEnd': 15,
                        'nStart': 14,
                        'sLineId': '#5643',
                        'sMessage': 'Confusion probable : “à” est une '
                                    'préposition. Pour le verbe '
                                    '“avoir”, écrivez “a”.',
                        'sRuleId': 'conf_a_à_verbe__b13_a2_0',
                        'sType': 'conf'},
                        {'URL': '',
                         'aColor': [89, 49, 129],
                         'aSuggestions': ['vérifier'],
                         'nEnd': 23,
                         'nStart': 16,
                         'sLineId': '#5643',
                         'sMessage': 'Le verbe devrait être à '
                                     'l’infinitif.',
                         'sRuleId': 'conf_a_à_verbe__b13_a3_0',
                         'sType': 'conf'}],
                    'lSpellingErrors': []},
                    {'iParagraph': 3,
                     'lGrammarErrors': [
                         {'URL': '',
                          'aColor': [89, 49, 129],
                          'aSuggestions': ['a'],
                          'nEnd': 15,
                          'nStart': 14,
                          'sLineId': '#5643',
                          'sMessage': 'Confusion probable : “à” est une '
                                      'préposition. Pour le verbe '
                                      '“avoir”, écrivez “a”.',
                          'sRuleId': 'conf_a_à_verbe__b13_a2_0',
                          'sType': 'conf'},
                         {'URL': '',
                          'aColor': [89, 49, 129],
                          'aSuggestions': ['vérifier'],
                          'nEnd': 23,
                          'nStart': 16,
                          'sLineId': '#5643',
                          'sMessage': 'Le verbe devrait être à '
                                      'l’infinitif.',
                          'sRuleId': 'conf_a_à_verbe__b13_a3_0',
                          'sType': 'conf'}],
                     'lSpellingErrors': []}],

            })
        spellchecker.create([doc])

        expected = [{
            'message': 'Confusion probable : “à” est une préposition. Pour le '
                       'verbe “avoir”, écrivez “a”.',
            'startOffset': 14,
            'suggestions': [{'text': 'a'}],
            'text': 'à',
            'type': 'grammar'},
            {'message': 'Le verbe devrait être à l’infinitif.',
             'startOffset': 16,
             'suggestions': [{'text': 'vérifier'}],
             'text': 'vérifié',
             'type': 'grammar'},
            {'message': 'Confusion probable : “à” est une préposition. Pour le '
                        'verbe “avoir”, écrivez “a”.',
             'startOffset': 55,
             'suggestions': [{'text': 'a'}],
             'text': 'à',
             'type': 'grammar'},
            {'message': 'Le verbe devrait être à l’infinitif.',
             'startOffset': 57,
             'suggestions': [{'text': 'vérifier'}],
             'text': 'vérifié',
             'type': 'grammar'}]

        self.assertEqual(doc['errors'], expected)

    @responses.activate
    def test_suggest(self):
        """Check that spelling suggestions are working"""

        doc = {
            "spellchecker": "grammalecte",
            "text": "fote",
            "suggestions": True,
        }
        spellchecker = get_resource_service('spellchecker')
        check_url = urljoin(TEST_URL, PATH_SUGGEST)
        responses.add(
            responses.POST, check_url,
            json={
                'suggestions': [
                    'faute',
                    'fauté',
                    'féauté',
                    'sotte',
                    'rote',
                    'roté',
                    'note',
                    'noté',
                    'lote',
                    'lotte']
            }
        )
        spellchecker.create([doc])

        self.assertEqual(doc, {
            'spellchecker': 'grammalecte',
            'suggestions': [
                {'text': 'faute'},
                {'text': 'fauté'},
                {'text': 'féauté'},
                {'text': 'sotte'},
                {'text': 'rote'},
                {'text': 'roté'},
                {'text': 'note'},
                {'text': 'noté'},
                {'text': 'lote'},
                {'text': 'lotte'}],
            'text': 'fote'})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.archive_history.service import ArchiveHistoryService, ArchiveHistoryResource
import logging
from superdesk import get_backend

log = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'archive_history'

    service = ArchiveHistoryService(endpoint_name, backend=get_backend())
    ArchiveHistoryResource(endpoint_name, app=app, service=service)

    app.on_updated_archive += service.on_item_updated

    app.on_archive_item_updated -= service.on_item_updated
    app.on_archive_item_updated += service.on_item_updated

    app.on_archive_item_deleted -= service.on_item_deleted
    app.on_archive_item_deleted += service.on_item_deleted

    # Store lock information to generate statistics on time spent editing items
    app.on_item_locked -= service.on_item_locked
    app.on_item_locked += service.on_item_locked

    app.on_item_unlocked -= service.on_item_unlocked
    app.on_item_unlocked += service.on_item_unlocked



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from flask import g
from superdesk.resource import Resource
from superdesk.services import BaseService

log = logging.getLogger(__name__)


class AuditResource(Resource):
    endpoint_name = 'audit'
    resource_methods = ['GET']
    item_methods = ['GET']
    schema = {
        'resource': {'type': 'string'},
        'action': {'type': 'string'},
        'audit_id': {'type': 'string'},
        'extra': {'type': 'dict'},
        'user': Resource.rel('users', False)
    }
    exclude = {endpoint_name, 'activity', 'dictionaries', 'macros', 'archive_history', 'formatters'}


class AuditService(BaseService):
    def on_generic_inserted(self, resource, docs):
        if resource in AuditResource.exclude:
            return

        user = getattr(g, 'user', None)
        if not user:
            if resource == 'auth':
                user_id = docs[0].get('user')
            else:
                return
        else:
            user_id = user.get('_id')

        if not len(docs):
            return

        audit = {
            'user': user_id,
            'resource': resource,
            'action': 'created',
            'extra': docs[0],
            'audit_id': self._extract_doc_id(docs[0])
        }

        self.post([audit])

    def on_generic_updated(self, resource, doc, original):
        if resource in AuditResource.exclude:
            return

        user = getattr(g, 'user', None)
        if not user:
            return

        audit = {
            'user': user.get('_id'),
            'resource': resource,
            'action': 'updated',
            'extra': doc,
            'audit_id': self._extract_doc_id(doc) if self._extract_doc_id(doc) else self._extract_doc_id(original)
        }
        if '_id' not in doc:
            audit['extra']['_id'] = original.get('_id', None)
        self.post([audit])

    def on_generic_deleted(self, resource, doc):
        if resource in AuditResource.exclude:
            return

        user = getattr(g, 'user', None)
        if not user:
            return

        audit = {
            'user': user.get('_id'),
            'resource': resource,
            'action': 'deleted',
            'extra': doc,
            'audit_id': self._extract_doc_id(doc)
        }
        self.post([audit])

    def _extract_doc_id(self, doc):
        """
        Given an audit item try to extract the id of the item that it relates to
        :param item:
        :return:
        """
        try:
            id = doc.get('_id', doc.get('guid', doc.get('item_id', doc.get('item', None))))
            # do not return an id for items that have a dictionary id
            if not isinstance(id, dict):
                return id
            else:
                None
        except Exception:
            return None
        return None



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import pytz
from superdesk.etree import etree
import html
from superdesk.io.feed_parsers.newsml_1_2 import NewsMLOneFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.errors import ParserError
from dateutil.parser import parse as date_parser
from flask import current_app as app
from apps.archive.common import format_dateline_to_locmmmddsrc


class ANANewsMLOneFeedParser(NewsMLOneFeedParser):
    """ANA specific NewsML parser.

    Feed Parser which can parse the ANA English feed basically it is in NewsML 1.2 format
    """

    NAME = 'ananewsml12'

    label = 'ANA NewsML Parser'

    def can_parse(self, xml):
        return xml.tag == 'NewsML'

    def parse(self, xml, provider=None):
        item = {}
        try:
            self.root = xml
            parsed_el = xml.find('NewsItem/NewsComponent/AdministrativeMetadata/Source/Party')
            if parsed_el is not None:
                item['original_source'] = parsed_el.attrib.get('FormalName', 'ANA')

            parsed_el = xml.find('NewsEnvelope/Priority')
            item['priority'] = self.map_priority(parsed_el.text if parsed_el is not None else None)

            self.parse_news_identifier(item, xml)
            self.parse_newslines(item, xml)
            self.parse_news_management(item, xml)

            parsed_el = xml.findall('NewsItem/NewsComponent/DescriptiveMetadata/Language')
            if parsed_el is not None:
                language = self.parse_attributes_as_dictionary(parsed_el)
                item['language'] = language[0]['FormalName'] if len(language) else ''

            subjects = xml.findall(
                'NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectDetail[@Scheme="IptcSubjectCodes"]')
            subjects += xml.findall(
                'NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectMatter[@Scheme="IptcSubjectCodes"]')
            subjects += xml.findall(
                'NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/Subject[@Scheme="IptcSubjectCodes"]')

            item['subject'] = self.format_subjects(subjects)

            item['body_html'] = html.unescape(etree.tostring(
                xml.find('NewsItem/NewsComponent/NewsComponent/ContentItem/DataContent'),
                encoding='unicode')).replace('<DataContent>', '').replace('</DataContent>', '').replace('<P>',
                                                                                                        '<p>').replace(
                '</P>', '</p>')

            item['body_html'] = item.get('body_html').replace('<p>© ΑΠΕ-ΜΠΕ ΑΕ. Τα πνευματικά δικαιώματα ανήκουν στο '
                                                              'ΑΠΕ-ΜΠΕ ΑΕ και παραχωρούνται σε συνδρομητές μόνον '
                                                              'για συγκεκριμένη χρήση.</p>', '').strip()
            parsed_el = xml.findall('NewsItem/NewsComponent/NewsComponent/ContentItem/Characteristics/Property')
            characteristics = self.parse_attribute_values(parsed_el, 'WordCount')
            item['word_count'] = characteristics[0] if len(characteristics) else None

            # Extract the city for setting into the dateline
            city = xml.find('NewsItem/NewsComponent/DescriptiveMetadata/Property[@FormalName="City"]').attrib.get(
                'Value')
            # Anglicise the greek for Athens if required
            city = 'Athens' if city == 'Αθήνα' else city
            country = xml.find('NewsItem/NewsComponent/DescriptiveMetadata/Property[@FormalName="Country"]').attrib.get(
                'Value')
            # Normalise the country code
            country = 'GR' if country == 'GRC' else country

            cities = app.locators.find_cities()
            located = [c for c in cities if c['city'] == city and c['country_code'] == country]
            if len(located) == 1:
                item['dateline']['located'] = located[0]
                item['dateline']['source'] = provider.get('source')
                item['dateline']['text'] = format_dateline_to_locmmmddsrc(item['dateline']['located'],
                                                                          item.get('dateline', {}).get('date'),
                                                                          provider.get('source'))
            return self.populate_fields(item)
        except Exception as ex:
            raise ParserError.newsmlOneParserError(ex, provider)

    def parse_newslines(self, item, tree):
        parsed_el = self.parse_elements(tree.find('NewsItem/NewsComponent/NewsLines'))

        # Set the date component of the dateline assuming that the date is in the Athens timezone
        local_dt = date_parser(parsed_el.get('DateLine', ''), fuzzy=True, dayfirst=True)
        local_tz = pytz.timezone('Europe/Athens')
        aus_dt = local_tz.localize(local_dt, is_dst=None)
        item.setdefault('dateline', {})
        item['dateline']['date'] = aus_dt.astimezone(pytz.utc)

        item['headline'] = parsed_el.get('HeadLine', '').strip()
        item['slugline'] = parsed_el.get('SlugLine', '')
        # The byline is not extracted as it seems to use the Greek character set

        return True

    def parse_news_management(self, item, tree):
        parsed_el = self.parse_elements(tree.find('NewsItem/NewsManagement'))
        item['urgency'] = int(parsed_el.get('Urgency', {}).get('FormalName', 3))
        item['versioncreated'] = self.datetime(parsed_el['ThisRevisionCreated'])
        item['firstcreated'] = self.datetime(parsed_el['FirstCreated'])
        item['pubstatus'] = (parsed_el['Status']['FormalName']).lower()

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, '%Y%m%dT%H%M%S%z')
        except ValueError:
            return super().datetime(string)


register_feed_parser(ANANewsMLOneFeedParser.NAME, ANANewsMLOneFeedParser())




# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
import logging
from lxml import etree  # noqa
from superdesk import etree as sd_etree
from lxml import html as lxml_html
from lxml.html import clean
from flask import current_app as app
import chardet

logger = logging.getLogger(__name__)


# This pattern matches http(s) links, numbers (1.000.000 or 1,000,000 or 1 000 000), regulars words,
# compound words (e.g. "two-done") or abbreviation (e.g. D.C.)
# If you modify please keep in sync with superdesk-client/core/scripts/apps/authoring/authoring/directives/WordCount.js
WORD_PATTERN = re.compile(r'https?:[^ ]*|([0-9]+[,. ]?)+|([\w]\.)+|[\w][\w-]*')


def get_text_word_count(text):
    """Get word count for given plain text.

    :param str text: text string
    :return int: word count
    """
    return sum(1 for word in WORD_PATTERN.finditer(text))


def get_text(markup, content='xml', lf_on_block=False, space_on_elements=False):
    """Get plain text version of (X)HTML or other XML element

    if the markup can't be parsed, it will be returned unchanged
    :param str markup: string to convert to plain text
    :param str content: 'xml' or 'html', as in parse_html
    :param bool lf_on_block: if True, add a line feed on block elements' tail
    :param bool space_on_elements: if True, add a space on each element's tail
        mainly used to count words with non HTML markup
    :return str: plain text version of markup
    """
    try:
        root = sd_etree.parse_html(
            markup,
            content=content,
            lf_on_block=lf_on_block,
            space_on_elements=space_on_elements)
        text = etree.tostring(root, encoding='unicode', method='text')
        return text
    except etree.ParseError:
        return markup


def get_word_count(markup, no_html=False):
    """Get word count for given html.

    :param str markup: xhtml (or other xml) markup
    :param bool no_html: set to True if xml param is not (X)HTML
        if True, a space will be added after each element to separate words.
        This avoid to have construct like <hl2>word</hl2><p>another</p> (like in NITF)
        being counted as one word.
    :return int: count of words inside the text
    """
    if no_html:
        return get_text_word_count(get_text(markup, content='xml', space_on_elements=True))
    else:
        return get_text_word_count(get_text(markup, content='html', lf_on_block=True))


def update_word_count(update, original=None):
    """Update word count if there was change in content.

    :param update: created/updated document
    :param original: original document if updated
    """
    if update.get('body_html'):
        update.setdefault('word_count', get_word_count(update.get('body_html')))
    else:
        # If the body is removed then set the count to zero
        if original and 'word_count' in original and 'body_html' in update:
            update['word_count'] = 0


def get_char_count(html):
    """Get character count for given html.

    :param html: html string to count
    :return int: count of chars inside the text
    """
    return len(get_text(html))


def get_par_count(html):
    try:
        elem = sd_etree.parse_html(html, content='html')
        return len([
            p for p in elem.iterfind('.//p')
            if p.text and len(p.text.strip()) > 0
        ])
    except ValueError as e:
        logger.warning(e)

    logger.warning('Failed to determine paragraph count from html: {}.'.format(html))
    return 0


def get_reading_time(html, word_count=None, language=None):
    """Get estimanted number of minutes to read a text

    Check https://dev.sourcefabric.org/browse/SDFID-118 for details

    :param str html: html content
    :param int word_count: number of words in the text
    :param str language: language of the text
    :return int: estimated number of minute to read the text
    """
    if language and language.startswith('ja'):
        return round(len(re.sub(r'[\s]', '', get_text(html))) / app.config['JAPANESE_CHARACTERS_PER_MINUTE'])
    if not word_count:
        word_count = get_word_count(html)
    reading_time_float = word_count / 250
    reading_time_minutes = int(reading_time_float)
    reading_time_rem_sec = int((reading_time_float - reading_time_minutes) * 60)
    if reading_time_rem_sec >= 30:
        reading_time_minutes += 1
    return reading_time_minutes


def sanitize_html(html):
    """Sanitize HTML

    :param str html: unsafe HTML markup
    :return str: sanitized HTML
    """
    if not html:
        return ""

    blacklist = ["script", "style", "head"]

    root_elem = lxml_html.fromstring(html)
    cleaner = clean.Cleaner(
        add_nofollow=False,
        kill_tags=blacklist
    )
    cleaned_xhtml = cleaner.clean_html(root_elem)

    safe_html = etree.tostring(cleaned_xhtml, encoding="unicode")

    # the following code is legacy (pre-lxml)
    if safe_html == ", -":
        return ""

    return safe_html


def decode(bytes_str):
    """Decode bytes value

    try to decode using UTF-8, or to detect encoding. Will ignore bad chars as a last resort
    @return (str): decoded string
    """
    try:
        return bytes_str.decode('utf-8')
    except UnicodeDecodeError:
        try:
            return bytes_str.decode(chardet.detect(bytes_str)['encoding'])
        except Exception:
            return bytes_str.decode('utf-8', 'ignore')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import urllib
import requests

from flask import current_app as app
from superdesk.errors import SuperdeskApiError
from flask_babel import _


class AlchemyKeywordsProvider():
    """Keyword provider that user the Alchemy API(http://www.alchemyapi.com/api)"""

    def __init__(self):
        self._http = requests.Session()

    def get_keywords(self, text):
        if not app.config['KEYWORDS_KEY_API']:
            raise SuperdeskApiError.notFoundError(_('AlchemyAPI key is not set'))

        params = {'apikey': app.config['KEYWORDS_KEY_API'],
                  'outputMode': 'json'}

        url = app.config['KEYWORDS_BASE_URL'] + '/text/TextGetRankedNamedEntities' + \
            '?' + urllib.parse.urlencode(params)

        values = {'text': text}

        result = ""
        try:
            result = self._http.post(url, data=values)
        except Exception as ex:
            raise SuperdeskApiError.internalError(_('Fail to connect to Alchemy service'), exception=ex)

        try:
            keywords = result.json()
            return keywords.get('entities', [])
        except Exception as ex:
            raise SuperdeskApiError.internalError(_('Fail to parse the response from Alchemy service'), exception=ex)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
import superdesk.publish

from datetime import timedelta
from eve.utils import config, ParsedRequest
from flask import current_app as app
from superdesk import get_resource_service
from superdesk.celery_task_utils import get_lock_id
from superdesk.errors import PublishHTTPPushClientError
from superdesk.lock import lock, unlock
from superdesk.celery_app import celery
from superdesk.utc import utcnow
from superdesk.profiling import ProfileManager
from .publish_queue import QueueState
from celery.exceptions import SoftTimeLimitExceeded

logger = logging.getLogger(__name__)

UPDATE_SCHEDULE_DEFAULT = {'seconds': 10}
PUBLISH_QUEUE = 'publish_queue'
STATE_PENDING = 'pending'


class PublishContent(superdesk.Command):
    """Deliver items from ``publish_queue to destinations.``

    Example:
    ::

        $ python manage.py publish:transmit

    """

    def run(self, provider_type=None):
        publish.apply_async(expires=10)


@celery.task(soft_time_limit=1800)
def publish():
    """Fetch items from publish queue as per the configuration, call the transmit function."""
    with ProfileManager('publish:transmit'):
        lock_name = get_lock_id("Transmit", "Articles")
        if not lock(lock_name, expire=1810):
            logger.info('Task: {} is already running.'.format(lock_name))
            return

        try:
            for priority in [True, False]:  # top priority first
                for retries in [False, True]:  # first publish pending, retries after
                    subs = get_queue_subscribers(priority=priority, retries=retries)
                    for sub in subs:
                        transmit_subscriber_items.apply_async(
                            (str(sub), ),
                            {'retries': retries, 'priority': priority},
                            queue=_get_queue(priority),
                        )
        except Exception:
            logger.exception('Task: {} failed.'.format(lock_name))
        finally:
            unlock(lock_name)


def _get_queue_lookup(retries=False, priority=None):
    priority_lookup = {'priority': priority if priority else {'$ne': True}}
    if retries:
        return {
            '$and': [
                {'state': QueueState.RETRYING.value},
                {'next_retry_attempt_at': {'$lte': utcnow()}},
                priority_lookup,
            ]
        }
    return {
        '$and': [
            {'state': QueueState.PENDING.value},
            priority_lookup,
        ]
    }


def get_queue_subscribers(retries=False, priority=None):
    lookup = _get_queue_lookup(retries, priority)
    return app.data.mongo.pymongo(resource=PUBLISH_QUEUE).db[PUBLISH_QUEUE].distinct('subscriber_id', lookup)


def get_queue_items(retries=False, subscriber_id=None, priority=None):
    lookup = _get_queue_lookup(retries, priority)
    if subscriber_id:
        lookup['$and'].append({'subscriber_id': subscriber_id})
    request = ParsedRequest()
    request.max_results = app.config.get('MAX_TRANSMIT_QUERY_LIMIT', 100)  # limit per subscriber now
    request.sort = '[("_created", 1), ("published_seq_num", 1)]'
    return get_resource_service(PUBLISH_QUEUE).get(req=request, lookup=lookup)


def _get_queue(priority=None):
    if priority and app.config.get('HIGH_PRIORITY_QUEUE_ENABLED'):
        return app.config['HIGH_PRIORITY_QUEUE']


@celery.task(soft_time_limit=600)
def transmit_subscriber_items(subscriber, retries=False, priority=None):
    lock_name = get_lock_id('Subscriber', 'Transmit', subscriber)
    is_async = get_resource_service('subscribers').is_async(subscriber)

    if not lock(lock_name, expire=610):
        return

    try:
        queue_items = get_queue_items(retries, subscriber, priority)
        for queue_item in queue_items:
            args = [queue_item[config.ID_FIELD]]
            kwargs = {'is_async': is_async}
            if is_async:
                transmit_item.apply_async(
                    args=args,
                    kwargs=kwargs,
                    queue=_get_queue(priority),
                )
            else:
                transmit_item.apply(args=args, kwargs=kwargs, throw=True)
    finally:
        unlock(lock_name)


@celery.task(soft_time_limit=300)
def transmit_item(queue_item_id, is_async=False):
    publish_queue_service = get_resource_service(PUBLISH_QUEUE)
    lock_name = get_lock_id('Transmit', str(queue_item_id))
    if is_async and not lock(lock_name, expire=310):
        return
    try:
        # check the status of the queue item
        queue_item = publish_queue_service.find_one(req=None, _id=queue_item_id)
        if queue_item.get('state') not in [QueueState.PENDING.value, QueueState.RETRYING.value]:
            logger.info('Transmit State is not pending/retrying for queue item: {}. It is in {}'.
                        format(queue_item.get(config.ID_FIELD), queue_item.get('state')))
            return

        log_msg = '_id: {_id}  item_id: {item_id}  state: {state} ' \
                  'item_version: {item_version} headline: {headline}'.format(**queue_item)

        # update the status of the item to in-progress
        queue_update = {'state': 'in-progress', 'transmit_started_at': utcnow()}
        publish_queue_service.patch(queue_item.get(config.ID_FIELD), queue_update)
        logger.info('Transmitting queue item {}'.format(log_msg))

        destination = queue_item['destination']
        transmitter = superdesk.publish.registered_transmitters[destination.get('delivery_type')]
        transmitter.transmit(queue_item)
        logger.info('Transmitted queue item {}'.format(log_msg))
    except Exception as e:
        logger.exception('Failed to transmit queue item {}'.format(log_msg))

        max_retry_attempt = app.config.get('MAX_TRANSMIT_RETRY_ATTEMPT')
        retry_attempt_delay = app.config.get('TRANSMIT_RETRY_ATTEMPT_DELAY_MINUTES')
        try:
            orig_item = publish_queue_service.find_one(req=None, _id=queue_item['_id'])
            updates = {config.LAST_UPDATED: utcnow()}

            if orig_item.get('retry_attempt', 0) < max_retry_attempt and \
                    not isinstance(e, PublishHTTPPushClientError):

                updates['retry_attempt'] = orig_item.get('retry_attempt', 0) + 1
                updates['state'] = QueueState.RETRYING.value
                updates['next_retry_attempt_at'] = utcnow() + timedelta(minutes=retry_attempt_delay)
            else:
                # all retry attempts exhausted marking the item as failed.
                updates['state'] = QueueState.FAILED.value

            publish_queue_service.system_update(orig_item.get(config.ID_FIELD), updates, orig_item)
        except Exception:
            logger.error('Failed to set the state for failed publish queue item {}.'.format(queue_item['_id']))
        if isinstance(e, SoftTimeLimitExceeded):
            raise
    finally:
        if is_async:
            unlock(lock_name, remove=True)


superdesk.command('publish:transmit', PublishContent())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk import get_resource_service
from superdesk.io import registry
from lxml import etree
import sys
from superdesk.metadata.item import ITEM_STATE


class ImportCommand(superdesk.Command):
    """Import articles into archives.

    Example:
    ::

        $ python manage.py xml:import ninjs data/sample.json

    """

    option_list = [
        superdesk.Option('parser', help="name of the feed parser"),
        superdesk.Option('path', help="path to the archive file to parse"),
        superdesk.Option('--profile', '-p', help="name of the profile to use (case sensitive"),
    ]

    def run(self, parser, path, profile):
        try:
            feed_parser = registry.registered_feed_parsers[parser]
        except KeyError:
            print("Can't find feed parser with this name")
            sys.exit(1)

        if profile is not None:
            content_types_service = get_resource_service("content_types")
            try:
                content_profile = content_types_service.find({'label': profile}).next()
            except StopIteration:
                print("Can't find content profile with this label")
                sys.exit(1)
            else:
                profile_id = content_profile['_id']

        with open(path, 'rb') as f:
            buf = f.read()
            buf = buf.replace(b'\r', b'&#13;')
            xml_parser = etree.XMLParser(recover=True)
            parsed = etree.fromstring(buf, xml_parser)
        articles = feed_parser.parse(parsed)
        updates = {ITEM_STATE: 'published'}
        if profile is not None:
            updates['profile'] = profile_id
        for article in articles:
            article.update(updates)
            article.setdefault('source', parser)
        archived_service = get_resource_service("archived")
        archived_service.post(articles)


superdesk.command('xml:import', ImportCommand())




import superdesk.schema as schema


class DefaultSchema(schema.Schema):
    """Default schema."""

    #: usageterms
    usageterms = schema.StringField()

    #: keywords
    keywords = schema.ListField()

    #: language
    language = schema.StringField()

    #: slugline
    slugline = schema.StringField(maxlength=24)

    #: item genre
    genre = schema.ListField()

    #: anpa take key
    anpa_take_key = schema.StringField()

    #: place where news happened
    place = schema.ListField()

    #: news item priority
    priority = schema.IntegerField()

    #: news item urgency
    urgency = schema.IntegerField()

    #: category
    anpa_category = schema.ListField()

    #: subject
    subject = schema.ListField(required=True, mandatory_in_list={'scheme': {}}, schema={
        'type': 'dict',
        'schema': {
            'name': {},
            'qcode': {},
            'scheme': {
                'type': 'string',
                'required': True,
                'nullable': True,
                'allowed': []
            },
            'service': {'nullable': True},
            'parent': {'nullable': True}
        }
    })

    #: company codes
    company_codes = schema.ListField()

    #: editorial note
    ednote = schema.StringField()

    #: authors
    authors = schema.ListField(schema={
        'type': 'dict',
        'schema': {
            'name': {'type': 'string'},
            'parent': {'type': 'string'},
            'role': {'type': 'string'},
        }
    })

    #: headline
    headline = schema.StringField(maxlength=64)

    #: sms version of an item
    sms = schema.StringField()

    #: item abstract
    abstract = schema.StringField(maxlength=160)

    #: byline
    byline = schema.StringField()

    #: dateline - info about where news was written
    dateline = schema.DictField()

    #: item content
    body_html = schema.StringField()

    #: item footer
    footer = schema.StringField()

    #: body footer
    body_footer = schema.StringField()

    #: item sign off info
    sign_off = schema.StringField()

    #: embedded media in the item
    feature_media = schema.MediaField()

    #: embedded media description
    media_description = schema.StringField()

    #: item attachments
    #: .. versionadded:: 1.29
    attachments = schema.ListField()


DEFAULT_SCHEMA = dict(DefaultSchema)


DEFAULT_EDITOR = {
    'slugline': {'order': 1, 'sdWidth': 'full', 'enabled': True},
    'keywords': {'order': 2, 'sdWidth': 'full', 'enabled': False},
    'language': {'order': 3, 'sdWidth': 'half', 'enabled': False},
    'usageterms': {'order': 4, 'sdWidth': 'full', 'enabled': False},
    'genre': {'order': 5, 'sdWidth': 'half', 'enabled': True},
    'anpa_take_key': {'order': 6, 'sdWidth': 'half', 'enabled': False},
    'place': {'order': 7, 'sdWidth': 'half', 'enabled': True},
    'priority': {'order': 8, 'sdWidth': 'quarter', 'enabled': True},
    'urgency': {'order': 9, 'sdWidth': 'quarter', 'enabled': True},
    'anpa_category': {'order': 10, 'sdWidth': 'full', 'enabled': True},
    'subject': {'order': 11, 'sdWidth': 'full', 'enabled': True},
    'company_codes': {'order': 12, 'sdWidth': 'full', 'enabled': False},
    'ednote': {'order': 13, 'sdWidth': 'full', 'enabled': True},
    'authors': {'order': 14, 'sdWidth': 'full', 'enabled': True},
    'headline': {'order': 15, 'formatOptions': [], 'enabled': True},
    'sms': {'order': 16, 'enabled': False},
    'abstract': {
        'order': 17,
        'formatOptions': ['bold', 'italic', 'underline', 'link'],
        'enabled': True
    },
    'byline': {'order': 18, 'enabled': True},
    'dateline': {'order': 19, 'enabled': True},
    'body_html': {
        'order': 20,
        'formatOptions': ['h2', 'bold', 'italic', 'underline', 'quote', 'link', 'embed', 'media'],
        'cleanPastedHTML': False,
        'enabled': True
    },
    'footer': {'order': 21, 'enabled': False},
    'body_footer': {'order': 22, 'enabled': False},
    'sign_off': {'order': 23, 'enabled': True},
    'feature_media': {'enabled': True},
    'media_description': {'enabled': True},
    'attachments': {'enabled': False},
}
